import { Component, OnDestroy, OnInit } from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { Subscription } from 'rxjs';
import { AppState } from 'src/app/app.reducers';
import { MessageService } from 'src/app/Shared/services/message.service';
import * as AuthAction from '../../actions';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss'],
})
export class LoginComponent implements OnInit, OnDestroy {
  username: FormControl;
  password: FormControl;
  loginForm: FormGroup;
  subscription: Subscription | null;

  constructor(
    private formBuilder: FormBuilder,
    private store: Store<AppState>,
    private router: Router,
    private messageService: MessageService
  ) {
    this.username = new FormControl('', [
      Validators.required,
      Validators.minLength(4),
    ]);

    this.password = new FormControl('', [
      Validators.required,
      Validators.maxLength(16),
    ]);

    this.loginForm = this.formBuilder.group({
      username: this.username,
      password: this.password,
    });

    this.subscription = this.store.select('auth').subscribe((auth) => {
      if (auth.logged) {
        if (!auth.authentication.canviPassword) {
          this.router.navigateByUrl('home');
        } else {
          this.router.navigateByUrl('auth/change-password');
        }
      }
      if (auth.error) {
        messageService.showHttpResponseError(
          'loginFeedback',
          auth.error.payload
        );
      }
    });
  }

  ngOnInit(): void {}

  ngOnDestroy() {
    if (this.subscription) this.subscription.unsubscribe();
  }

  login(): void {
    if (!this.loginForm.invalid) {
      this.store.dispatch(
        AuthAction.login({ credentials: this.loginForm.value })
      );
    }
  }
}
