import { HttpClientTestingModule } from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Router } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { provideMockStore } from '@ngrx/store/testing';
import { of } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { ArticleDTO } from '../../model/article.dto';
import { ArticleService } from '../../services/ordenances.service';
import { OrdenancesFormComponent } from './ordenances-form.component';
//mock lista de categorías

const articleUpdate: ArticleDTO = {
  karticle: 1,
  codi: '10',
  descripcio: 'ARTICULO 100',
  importSancio: 100,
  importMinim: null,
  importMaxim: null,
};

const articleInsert: ArticleDTO = {
  codi: '10',
  descripcio: 'ARTICULO 100',
  importSancio: 100,
  importMinim: null,
  importMaxim: null,
};

const initialAuthState = {
  authentication: new AuthDTO(
    'admin',
    'acces_token',
    'refresh_token',
    false,
    []
  ),
  loggedIn: false,
  processing: false,
  logged: true,
  passwordChanged: false,
  error: null,
};

const initialState = { auth: initialAuthState };

class TemporalComponentForRoutes {}
describe('OrdenancesFormComponent', () => {
  let component: OrdenancesFormComponent;
  let fixture: ComponentFixture<OrdenancesFormComponent>;

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [
        HttpClientTestingModule,
        ReactiveFormsModule,
        FormsModule,
        RouterTestingModule.withRoutes([
          {
            path: 'ordenances/ordenances-list',
            component: TemporalComponentForRoutes,
          },
        ]),
      ],
      declarations: [OrdenancesFormComponent],
      providers: [ArticleService, provideMockStore({ initialState })],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    }).compileComponents()
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(OrdenancesFormComponent);
    //    store = TestBed.inject(MockStore);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('OrdenancesFormComponent creado correctamente', () => {
    expect(component).toBeTruthy();
  });

  it('Se llama a la la funcion saveArticle del servicio ArticleService al actualizar una ordenanza', () => {
    const articleService = fixture.debugElement.injector.get(ArticleService);

    const spy = spyOn(articleService, 'saveArticle').and.returnValue(
      of(articleUpdate)
    );

    component.karticle = '1';
    component.accio = 'update';
    component.codi.setValue(articleUpdate.codi);
    component.descripcio.setValue(articleUpdate.descripcio);
    component.import_sancio.setValue(articleUpdate.importSancio);
    component.import_minim.setValue(articleUpdate.importMinim);
    component.import_maxim.setValue(articleUpdate.importMaxim);

    component['saveArticle']();
    expect(spy).toHaveBeenCalled();
    expect(component.article).toEqual(articleUpdate);
  });

  it('Se llama a la la funcion addArticle del servicio ArticleService al añadir una ordenanza', () => {
    const articleService = fixture.debugElement.injector.get(ArticleService);

    const spy = spyOn(articleService, 'addArticle').and.returnValue(
      of(articleInsert)
    );
    component.accio = 'insert';
    component.codi.setValue(articleInsert.codi);
    component.descripcio.setValue(articleInsert.descripcio);
    component.import_sancio.setValue(articleInsert.importSancio);
    component.import_minim.setValue(articleInsert.importMinim);
    component.import_maxim.setValue(articleInsert.importMaxim);
    component['saveArticle']();
    expect(spy).toHaveBeenCalled();
    expect(component.article).toEqual(articleInsert);
  });

  it('Se llama a la la funcion deleteArticle del servicio ArticleService al eliminar una ordenanza', () => {
    const articleService = fixture.debugElement.injector.get(ArticleService);

    const spy = spyOn(articleService, 'deleteArticle').and.returnValue(
      of(Object)
    );
    component.karticle = '1';
    component.accio = 'delete';
    component.codi.setValue(articleUpdate.codi);
    component.descripcio.setValue(articleUpdate.descripcio);
    component.import_sancio.setValue(articleUpdate.importSancio);
    component.import_minim.setValue(articleUpdate.importMinim);
    component.import_maxim.setValue(articleUpdate.importMaxim);
    component['saveArticle']();
    expect(spy).toHaveBeenCalled();
  });

  /*TEST4: valide que se lanza el navigateByUrl con el argumento correcto cuando creamos una categoría*/
  it('Navegamos correctamente cunado pulsamos exit', () => {
    const router = TestBed.inject(Router);
    const spy = spyOn(router, 'navigateByUrl');
    component.exit();
    expect(spy).toHaveBeenCalledWith('ordenances/ordenances-list');
  });
});
