import { HttpClientTestingModule } from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Router } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { provideMockStore } from '@ngrx/store/testing';
import { of } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { ArticleDTO } from '../../model/article.dto';
import { ArticleService } from '../../services/ordenances.service';
import { OrdenancesListComponent } from './ordenances-list.component';

const article: ArticleDTO = new ArticleDTO(
  1,
  '10.1',
  'ARTICULO 1',
  100,
  null,
  null
);

const articleList: ArticleDTO[] = [
  new ArticleDTO(1, '10.1', 'ARTICULO 1', 100, null, null),
  new ArticleDTO(2, '10.2', 'ARTICULO 2', 200, null, null),
  new ArticleDTO(3, '10.3.b', 'ARTICULO 3', null, 100, 500),
];

const initialAuthState = {
  authentication: new AuthDTO(
    'admin',
    'acces_token',
    'refresh_token',
    false,
    []
  ),
  loggedIn: false,
  processing: false,
  logged: true,
  passwordChanged: false,
  error: null,
};

const initialState = { auth: initialAuthState };

class TemporalComponentForRoutes {}
describe('OrdenancesListComponent', () => {
  let component: OrdenancesListComponent;
  let fixture: ComponentFixture<OrdenancesListComponent>;

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [
        HttpClientTestingModule,
        ReactiveFormsModule,
        FormsModule,
        RouterTestingModule.withRoutes([
          {
            path: 'ordenances/ordenances-form/insert/',
            component: TemporalComponentForRoutes,
          },
          {
            path: 'ordenances/ordenances-form/update/:karticle',
            component: TemporalComponentForRoutes,
          },
          {
            path: 'ordenances/ordenances-form/delete/:karticle',
            component: TemporalComponentForRoutes,
          },
        ]),
      ],
      declarations: [OrdenancesListComponent],
      providers: [provideMockStore({ initialState })],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    }).compileComponents()
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(OrdenancesListComponent);
    //    store = TestBed.inject(MockStore);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('UsersListComponent creado correctamente', () => {
    expect(component).toBeTruthy();
  });

  it('filterByName funciona correctamente desde una suscripción', () => {
    const articleService = fixture.debugElement.injector.get(ArticleService);

    const spy = spyOn(articleService, 'filterByName').and.returnValue(
      of(articleList)
    );
    component.descripcio.setValue('ARTICULO');
    component['filterByName']();
    expect(spy).toHaveBeenCalled();
    expect(component.articles).toBe(articleList);
  });

  it('filterByCodi funciona correctamente desde una suscripción', () => {
    const articleService = fixture.debugElement.injector.get(ArticleService);

    const spy = spyOn(articleService, 'filterByCode').and.returnValue(
      of(articleList)
    );
    component.codi.setValue('10');
    component['filterByCode']();
    expect(spy).toHaveBeenCalled();
    expect(component.articles).toBe(articleList);
  });

  it('Navegamos correctamente cunado queremos añadir una ordenanza', () => {
    const router = TestBed.inject(Router);
    const spy = spyOn(router, 'navigateByUrl');
    component.afegirArticle();
    expect(spy).toHaveBeenCalledWith('ordenances/ordenances-form/insert/');
  });

  it('Navegamos correctamente cunado queremos actualizar una Ordenanza', () => {
    const router = TestBed.inject(Router);
    const spy = spyOn(router, 'navigateByUrl');
    component.updateArticle(article);
    expect(spy).toHaveBeenCalledWith(
      'ordenances/ordenances-form/update/' + article.karticle
    );
  });

  it('Navegamos correctamente cunado queremos eliminar una Ordenanza', () => {
    const router = TestBed.inject(Router);
    const spy = spyOn(router, 'navigateByUrl');
    component.deleteArticle(article);
    expect(spy).toHaveBeenCalledWith(
      'ordenances/ordenances-form/delete/' + article.karticle
    );
  });
});
