import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { apiconfig } from 'src/app/Shared/model/apitool';
import { ArticleDTO } from '../model/article.dto';

@Injectable({
  providedIn: 'root',
})
export class ArticleService {
  private urlApi: string;

  constructor(private http: HttpClient) {
    this.urlApi = apiconfig.apiurl + 'article';
  }

  getArticle(karticle: string): Observable<ArticleDTO> {
    return this.http.get<ArticleDTO>(this.urlApi + '/' + karticle);
  }

  addArticle(article: ArticleDTO): Observable<ArticleDTO> {
    return this.http.post<ArticleDTO>(this.urlApi, article);
  }

  saveArticle(article: ArticleDTO): Observable<ArticleDTO> {
    return this.http.put<ArticleDTO>(this.urlApi, article);
  }

  deleteArticle(article: ArticleDTO): Observable<Object> {
    return this.http.delete(this.urlApi + '/' + article.karticle);
  }

  filterByName(filter: string): Observable<ArticleDTO[]> {
    return this.http.get<ArticleDTO[]>(
      this.urlApi + '/filterByName?filter=' + filter
    );
  }

  filterByCode(code: string): Observable<ArticleDTO[]> {
    return this.http.get<ArticleDTO[]>(
      this.urlApi + '/filterByCode?filter=' + code
    );
  }
}
