import { TextFieldModule } from '@angular/cdk/text-field';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { MatSlideToggleModule } from '@angular/material/slide-toggle';
import { provideMockStore } from '@ngrx/store/testing';
import { of } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { ActaDTO } from '../../model/acta.dto';
import { IdentificacioDTO } from '../../model/identificacio.dto';
import { NotificacioDTO } from '../../model/notificacio.dto';
import { PaisDTO } from '../../model/pais.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { PersonaService } from '../../services/persona.service';
import { ActaComponent } from './acta.component';

const denunciat = new IdentificacioDTO();
const denunciant = new IdentificacioDTO();

const initialAuthState = {
  authentication: new AuthDTO(
    'admin',
    'acces_token',
    'refresh_token',
    false,
    []
  ),
  loggedIn: false,
  processing: false,
  logged: true,
  passwordChanged: false,
  error: null,
};

const initialState = { auth: initialAuthState };

describe('ActaComponent', () => {
  let component: ActaComponent;
  let fixture: ComponentFixture<ActaComponent>;
  let menuSancioService: MenuSancioService;

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [
        HttpClientTestingModule,
        ReactiveFormsModule,
        FormsModule,
        MatAutocompleteModule,
        MatSlideToggleModule,
        TextFieldModule,
      ],
      declarations: [ActaComponent],
      providers: [provideMockStore({ initialState })],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    }).compileComponents()
  );

  beforeEach(() => {
    menuSancioService = TestBed.inject(MenuSancioService);
    fixture = TestBed.createComponent(ActaComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('ActaComponent creado correctamente', () => {
    expect(component).toBeTruthy();
  });

  it('findDenunciatByNif funciona correctamente desde una suscripción', () => {
    const personaService = fixture.debugElement.injector.get(PersonaService);

    const spy = spyOn(personaService, 'findByNif').and.returnValue(
      of(denunciat)
    );
    denunciat.document = '12345678Z';
    component.dni.setValue(denunciat.document);
    component['findByNif']();
    expect(spy).toHaveBeenCalled();
    expect(component.acta.denunciat).toBe(denunciat);
  });

  it('findDenunciantByNif funciona correctamente desde una suscripción', () => {
    const personaService = fixture.debugElement.injector.get(PersonaService);

    const spy = spyOn(personaService, 'findByNif').and.returnValue(
      of(denunciant)
    );
    denunciant.document = '12345678Z';
    component.denu_dni.setValue(denunciant.document);
    component['findDenunciantByNif']();
    expect(spy).toHaveBeenCalled();
    expect(component.acta.denunciant).toBe(denunciant);
  });

  it('findByDenunciatByPassaport funciona correctamente desde una suscripción', () => {
    const personaService = fixture.debugElement.injector.get(PersonaService);

    const spy = spyOn(personaService, 'findByPassaport').and.returnValue(
      of(denunciat)
    );
    denunciat.passaport = '1234567890';
    component.passaport.setValue(denunciat.passaport);
    component['findByPassaport']();
    expect(spy).toHaveBeenCalled();
    expect(component.acta.denunciat).toBe(denunciat);
  });

  it('findDenunciantByPassaport funciona correctamente desde una suscripción', () => {
    const personaService = fixture.debugElement.injector.get(PersonaService);

    const spy = spyOn(personaService, 'findByPassaport').and.returnValue(
      of(denunciant)
    );
    denunciant.passaport = '1234567890';
    component.denu_passaport.setValue(denunciant.passaport);
    component['findDenunciantByPassaport']();
    expect(spy).toHaveBeenCalled();
    expect(component.acta.denunciant).toBe(denunciant);
  });

  it('isValidForm devuelve que el formulario no es valido con datos incorrectos', () => {
    const isValidForm: boolean = component.isValidForm();
    expect(isValidForm).toBe(false);
  });

  it('isValidForm devuelve que el formulario  es valido con datos correctos', () => {
    const valid_denunciat: IdentificacioDTO = {
      document: '12345678Z',
      nom: 'PRUEBA',
      llinatge1: 'TEST1',
      llinatge2: 'TEST2',
      tpdocument: 'NIF',
      passaport: '',
      nomComplet: '',
      tppersona: '',
      sexe: '',
      telefon: '',
      email: '',
      nacionalitat: new PaisDTO(),
      dencomerc: '',
      activitat: '',
      domnotif: new NotificacioDTO(),
    };

    const actaValida = new ActaDTO();

    actaValida.descripcio = 'SANCIÓN DE PRUEBA';
    actaValida.denunciat = valid_denunciat;
    menuSancioService.setActa(actaValida);

    const fixture_2 = TestBed.createComponent(ActaComponent);
    const component_2 = fixture_2.componentInstance;
    fixture_2.detectChanges();

    const isValidForm: boolean = component_2.isValidForm();
    expect(isValidForm).toBe(true);
  });
});
