import { CdkTextareaAutosize } from '@angular/cdk/text-field';
import { formatDate } from '@angular/common';
import { HttpErrorResponse } from '@angular/common/http';
import { Component, NgZone, OnDestroy, OnInit, ViewChild } from '@angular/core';
import {
  AbstractControl,
  FormBuilder,
  FormControl,
  FormGroup,
  ValidationErrors,
  ValidatorFn,
  Validators,
} from '@angular/forms';
import { MatSelectChange } from '@angular/material/select';
import { Observable, Subscription } from 'rxjs';
import { map, startWith, take } from 'rxjs/operators';
import { ArticleDTO } from 'src/app/Modules/ordenances/model/article.dto';
import { IdentificacioDTO } from 'src/app/Modules/sancio/model/identificacio.dto';
import { MessageService } from 'src/app/Shared/services/message.service';
import {
  documentDenunciantRequiredValidator,
  documentFormatValidator,
  documentRequiredValidator,
  getTpDocument,
} from 'src/app/Shared/validators/document.validator';
import {
  nameDenunciantRequiredValidator,
  nameRequiredValidator,
} from 'src/app/Shared/validators/name.validator';
import { ActaDTO } from '../../model/acta.dto';
import { NotificacioDTO } from '../../model/notificacio.dto';
import { PaisDTO } from '../../model/pais.dto';
import { CadastreStorageService } from '../../services/cadastre-storage.service';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { PersonaService } from '../../services/persona.service';
import { SancioStorageService } from '../../services/sancio-storage.service';
@Component({
  selector: 'app-acta',
  templateUrl: './acta.component.html',
  styleUrls: ['./acta.component.scss'],
})
export class ActaComponent implements OnInit, OnDestroy {
  acta: ActaDTO;
  articlesLlista: ArticleDTO[];
  paisosLlista: PaisDTO[];

  actaForm: FormGroup;
  denunciatForm: FormGroup;
  denunciantForm: FormGroup;
  formValidated: boolean;

  articles: FormControl;
  filteredArticles: Observable<ArticleDTO[]>;
  descripcio: FormControl;

  dni: FormControl;
  passaport: FormControl;
  nom: FormControl;
  llinatge1: FormControl;
  llinatge2: FormControl;
  dencomercial: FormControl;
  activitat: FormControl;
  telefon: FormControl;
  sexe: FormControl;
  email: FormControl;
  data: FormControl;

  enableDenunciant: FormControl;

  denu_dni: FormControl;
  denu_passaport: FormControl;
  denu_nom: FormControl;
  denu_llinatge1: FormControl;
  denu_llinatge2: FormControl;
  denu_telefon: FormControl;
  denu_email: FormControl;
  denu_dencomercial: FormControl;
  denu_activitat: FormControl;

  old_document: string;
  old_passaport: string;

  old_denu_document: string;
  old_denu_passaport: string;

  pais: FormControl;
  filteredPaisos: Observable<PaisDTO[]>;

  denu_pais: FormControl;
  denu_filteredPaisos: Observable<PaisDTO[]>;

  article: ArticleDTO;

  subscription: Subscription;

  @ViewChild('autosize', { static: false })
  autosize: CdkTextareaAutosize | undefined;

  triggerResize() {
    // Wait for changes to be applied, then trigger textarea resize.
    this._ngZone.onStable.pipe(take(1)).subscribe(() => {
      if (this.autosize) this.autosize.resizeToFitContent(true);
    });
  }

  constructor(
    private formBuilder: FormBuilder,
    private cadastreStorageService: CadastreStorageService,
    private sancioStorageService: SancioStorageService,
    private personaService: PersonaService,
    private messageService: MessageService,
    private menuSancioService: MenuSancioService,
    private _ngZone: NgZone
  ) {
    this.formValidated = false;
    this.acta = menuSancioService.getActa();
    this.subscription = menuSancioService.change.subscribe(() => {
      menuSancioService.validated.emit(
        this.acta.kacta ? true : this.isValidForm()
      );
    });
    this.articlesLlista = [];
    this.paisosLlista = [];

    this.article = this.acta.article;

    this.articles = new FormControl(
      this.acta.article.descripcio,
      this.articleValidator
    );
    this.filteredArticles = this.articles.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterArticles(value))
    );
    this.descripcio = new FormControl(this.acta.descripcio);

    let denunciat: IdentificacioDTO = this.acta.denunciat;
    this.old_document = denunciat.document;
    this.old_passaport = denunciat.passaport;
    this.dni = new FormControl(denunciat.document, [documentFormatValidator]);
    this.passaport = new FormControl(
      denunciat.passaport,
      Validators.maxLength(20)
    );
    this.nom = new FormControl(denunciat.nom, Validators.maxLength(25));
    this.llinatge1 = new FormControl(
      denunciat.llinatge1,
      Validators.maxLength(25)
    );
    this.llinatge2 = new FormControl(
      denunciat.llinatge2,
      Validators.maxLength(25)
    );
    this.telefon = new FormControl(denunciat.telefon, Validators.maxLength(12));
    this.email = new FormControl(denunciat.email, [
      Validators.email,
      Validators.maxLength(32),
    ]);

    this.data = new FormControl(denunciat.dataneixement);
    this.sexe = new FormControl(denunciat.sexe);
    this.dencomercial = new FormControl(
      denunciat.dencomerc,
      Validators.maxLength(60)
    );
    this.activitat = new FormControl(
      denunciat.activitat,
      Validators.maxLength(60)
    );

    this.pais = new FormControl(
      !denunciat.nacionalitat ? '' : denunciat.nacionalitat.nom
    );
    this.filteredPaisos = this.pais.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterPaisos(value))
    );

    this.denunciatForm = this.formBuilder.group(
      {
        document: this.dni,
        passaport: this.passaport,
        nom: this.nom,
        llinatge1: this.llinatge1,
        llinatge2: this.llinatge2,
        nacionalitat: this.pais,
        telefon: this.telefon,
        email: this.email,
        sexe: this.sexe,
        dataneixement: this.data,
        dencomerc: this.dencomercial,
        activitat: this.activitat,
      },
      { validators: [documentRequiredValidator, nameRequiredValidator] }
    );

    if (!this.acta.denunciant) this.acta.denunciant = new IdentificacioDTO();
    let denunciant: IdentificacioDTO = this.acta.denunciant;

    this.enableDenunciant = new FormControl(denunciant.document !== '');

    this.old_denu_document = denunciant.document;
    this.old_denu_passaport = denunciant.passaport;

    this.denu_dni = new FormControl(denunciant.document, [
      documentFormatValidator,
    ]);
    this.denu_passaport = new FormControl(
      denunciant.passaport,
      Validators.maxLength(20)
    );
    this.denu_nom = new FormControl(denunciant.nom, Validators.maxLength(25));
    this.denu_llinatge1 = new FormControl(
      denunciant.llinatge1,
      Validators.maxLength(25)
    );
    this.denu_llinatge2 = new FormControl(
      denunciant.llinatge2,
      Validators.maxLength(25)
    );
    this.denu_telefon = new FormControl(
      denunciant.telefon,
      Validators.maxLength(12)
    );
    this.denu_email = new FormControl(denunciant.email, [
      Validators.email,
      Validators.maxLength(32),
    ]);

    this.denu_pais = new FormControl(
      !denunciant.nacionalitat ? '' : denunciant.nacionalitat.nom
    );
    this.denu_filteredPaisos = this.denu_pais.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterPaisos(value))
    );

    this.denu_dencomercial = new FormControl(
      denunciat.dencomerc,
      Validators.maxLength(60)
    );
    this.denu_activitat = new FormControl(
      denunciat.activitat,
      Validators.maxLength(60)
    );

    this.denunciantForm = this.formBuilder.group(
      {
        document: this.denu_dni,
        passaport: this.denu_passaport,
        nom: this.denu_nom,
        llinatge1: this.denu_llinatge1,
        llinatge2: this.denu_llinatge2,
        nacionalitat: this.denu_pais,
        telefon: this.denu_telefon,
        email: this.denu_email,
        dencomerc: this.denu_dencomercial,
        activitat: this.denu_activitat,
      },
      {
        validators: [
          documentDenunciantRequiredValidator,
          nameDenunciantRequiredValidator,
        ],
      }
    );

    this.actaForm = this.formBuilder.group(
      {
        article: this.articles,
        descripcio: this.descripcio,
        denunciat: this.denunciatForm,
        denunciant: this.denunciantForm,
        enableDenunciant: this.enableDenunciant,
      },
      { validators: [this.descripcioRequiredValidator] }
    );

    if (this.acta.kacta) {
      this.actaForm.disable();
    }

    this.initSelects();
  }

  hasDenunciant(): boolean {
    return this.enableDenunciant.value;
  }

  isDenunciatJuridica(): boolean {
    return getTpDocument(this.dni.value.toUpperCase()) === 'CIF';
  }

  isDenunciantJuridica(): boolean {
    return getTpDocument(this.denu_dni.value.toUpperCase()) === 'CIF';
  }

  ngOnInit(): void {}

  private initSelects() {
    this.sancioStorageService.getArticles(false).subscribe(
      (articles: ArticleDTO[]) => {
        this.articlesLlista = articles;
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError('sancioFeedback', error)
    );
    this.cadastreStorageService.getPaisos(false).subscribe(
      (paisos: PaisDTO[]) => {
        this.paisosLlista = paisos;
        if (this.acta.denunciat.nacionalitat) {
          this.pais.setValue(this.acta.denunciat.nacionalitat.nom);
        }
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError('sancioFeedback', error)
    );
  }

  private filterPaisos(value: string): PaisDTO[] {
    const paisosTmp: PaisDTO[] = this.paisosLlista.filter((pais) =>
      pais.nom.toUpperCase().includes(value.toUpperCase())
    );
    return paisosTmp;
  }

  private filterArticles(value: string): ArticleDTO[] {
    const articleTmp: ArticleDTO[] = this.articlesLlista.filter((article) =>
      article.descripcio.toUpperCase().includes(value.toUpperCase())
    );
    return articleTmp;
  }

  changeArticle(event: MatSelectChange) {
    let karticle = event.value;
    this.article = this.articlesLlista.filter(
      (article) => article.karticle === Number(karticle)
    )[0];
  }

  showInfo() {}

  private buildPais(): PaisDTO {
    const pais = this.paisosLlista.filter(
      (pais) => pais.nom === this.pais.value.toUpperCase()
    )[0];
    return pais ? pais : new PaisDTO();
  }

  private buildDenunciantPais(): PaisDTO {
    const pais = this.paisosLlista.filter(
      (pais) => pais.nom === this.denu_pais.value.toUpperCase()
    )[0];
    return pais ? pais : new PaisDTO();
  }

  private buildArticle(): ArticleDTO {
    const article = this.articlesLlista.filter(
      (article) => article.descripcio === this.articles.value.toUpperCase()
    )[0];
    return article ? article : new ArticleDTO(0, '', '', null, null, null);
  }

  findByNif(): void {
    if (
      !this.dni.invalid &&
      this.dni.value !== '' &&
      this.old_document !== this.dni.value
    ) {
      this.old_document = this.dni.value;
      const spers: Observable<IdentificacioDTO> = this.personaService.findByNif(
        this.dni.value
      );
      spers.pipe(take(1));
      spers.subscribe(
        (denunciat) => {
          this.buildDenunciat(denunciat);
        },
        (error: HttpErrorResponse) =>
          this.messageService.showHttpResponseError('sancioFeedback', error)
      );
    }
  }

  findDenunciantByNif(): void {
    if (
      !this.denu_dni.invalid &&
      this.denu_dni.value !== '' &&
      this.old_denu_document !== this.denu_dni.value
    ) {
      this.old_denu_document = this.denu_dni.value;
      const spers: Observable<IdentificacioDTO> = this.personaService.findByNif(
        this.denu_dni.value
      );
      spers.pipe(take(1));
      spers.subscribe(
        (denunciant) => {
          this.buildDenunciant(denunciant);
        },
        (error: HttpErrorResponse) =>
          this.messageService.showHttpResponseError('sancioFeedback', error)
      );
    }
  }

  findByPassaport(): void {
    if (
      this.dni.value === '' &&
      this.passaport.value !== '' &&
      this.old_passaport !== this.passaport.value
    ) {
      this.old_passaport = this.passaport.value;
      const spers: Observable<IdentificacioDTO> =
        this.personaService.findByPassaport(this.passaport.value);
      spers.pipe(take(1));
      spers.subscribe(
        (denunciat) => {
          this.buildDenunciat(denunciat);
        },
        (error) =>
          this.messageService.showHttpResponseError('sancioFeedback', error)
      );
    }
  }

  findDenunciantByPassaport(): void {
    if (
      this.denu_dni.value === '' &&
      this.denu_passaport.value !== '' &&
      this.old_denu_passaport !== this.denu_passaport.value
    ) {
      this.old_denu_passaport = this.denu_passaport.value;
      const spers: Observable<IdentificacioDTO> =
        this.personaService.findByPassaport(this.denu_passaport.value);
      spers.pipe(take(1));
      spers.subscribe(
        (denunciant) => {
          this.buildDenunciant(denunciant);
        },
        (error: HttpErrorResponse) =>
          this.messageService.showHttpResponseError('sancioFeedback', error)
      );
    }
  }

  private buildDenunciat(denunciat: IdentificacioDTO): void {
    if (denunciat) {
      this.acta.denunciat = denunciat;
      if (!denunciat.domnotif)
        this.acta.denunciat.domnotif = new NotificacioDTO();
      this.passaport.setValue(denunciat.passaport);
      this.nom.setValue(denunciat.nom);
      this.llinatge1.setValue(denunciat.llinatge1);
      this.llinatge2.setValue(denunciat.llinatge2);
      this.dencomercial.setValue(denunciat.dencomerc);
      this.activitat.setValue(denunciat.activitat);
      this.pais.setValue(denunciat.nacionalitat.nom);
      this.telefon.setValue(denunciat.telefon);
      this.email.setValue(denunciat.email);
      this.sexe.setValue(denunciat.sexe);
      if (denunciat.dataneixement) {
        let tdata: Date = new Date(
          denunciat.dataneixement.toString().substring(0, 10)
        );
        let fdata: string = formatDate(tdata, 'yyyy-MM-dd', 'en');
        this.data.setValue(fdata);
      } else this.data.setValue(null);
    } else this.cleanDenunciat();
  }

  private buildDenunciant(denunciant: IdentificacioDTO): void {
    if (denunciant) {
      this.acta.denunciant = denunciant;
      this.denu_passaport.setValue(denunciant.passaport);
      this.denu_nom.setValue(denunciant.nom);
      this.denu_llinatge1.setValue(denunciant.llinatge1);
      this.denu_llinatge2.setValue(denunciant.llinatge2);
      this.denu_dencomercial.setValue(denunciant.dencomerc);
      this.denu_activitat.setValue(denunciant.activitat);
      this.denu_pais.setValue(denunciant.nacionalitat.nom);
      this.denu_telefon.setValue(denunciant.telefon);
      this.denu_email.setValue(denunciant.email);
    } else this.cleanDenunciant();
  }

  changeSexe(sexe: string): void {
    this.sexe.setValue(sexe);
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  cleanDenunciat() {
    this.acta.denunciat = new IdentificacioDTO();
    this.nom.setValue('');
    this.llinatge1.setValue('');
    this.llinatge2.setValue('');
    this.telefon.setValue('');
    this.email.setValue('');
    this.dencomercial.setValue('');
    this.activitat.setValue('');
    this.pais.setValue(this.acta.denunciat.nacionalitat.nom);
    this.sexe.setValue('');
    this.data.setValue(null);
  }

  cleanAllDenunciant() {
    this.old_denu_document = '';
    this.old_denu_passaport = '';
    this.denu_dni.setValue('');
    this.denu_nom.setValue('');
    this.cleanDenunciant();
  }

  cleanDenunciant() {
    this.acta.denunciant = new IdentificacioDTO();
    this.denu_nom.setValue('');
    this.denu_llinatge1.setValue('');
    this.denu_llinatge2.setValue('');
    this.denu_telefon.setValue('');
    this.denu_email.setValue('');
    this.denu_dencomercial.setValue('');
    this.denu_activitat.setValue('');
    this.denu_pais.setValue(this.acta.denunciant.nacionalitat.nom);
  }

  //https://stackoverflow.com/questions/32260082/how-to-trigger-form-validators-in-angular2
  public isValidForm(): boolean {
    this.actaForm.markAllAsTouched();
    this.formValidated = true;
    if (!this.actaForm.invalid) {
      this.acta = {
        ...this.acta,
        article: this.buildArticle(),
        descripcio: this.descripcio.value,
        denunciat: {
          ...this.acta.denunciat,
          ...this.denunciatForm.value,
          tpdocument: this.getTpDocumentDenunciat(),
          dataneixement:
            this.data.value === '' ? null : new Date(this.data.value),
          nacionalitat: this.buildPais(),
        },
        denunciant: this.enableDenunciant.value
          ? {
              ...this.acta.denunciant,
              ...this.denunciantForm.value,
              tpdocument: this.getTpDocumentDenunciant(),
              nacionalitat: this.buildDenunciantPais(),
            }
          : null,
      };
      this.menuSancioService.setActa(this.acta);
      //      this.store.dispatch(updateActa({ acta: this.acta }));
    } else {
      Object.keys(this.actaForm.controls).forEach((key) => {
        // Recursive call of the FormGroup fields
        const controlErrors: ValidationErrors | null =
          this.actaForm.controls[key].errors;
        if (controlErrors !== null) {
          Object.keys(controlErrors).forEach((keyError) => {
            console.log('Control name:' + key);
            console.log('error name:' + keyError);
          });
        }
      });
    }

    return !this.actaForm.invalid;
  }

  private getTpDocumentDenunciat(): string {
    return this.dni.value === '' ? 'PAS' : getTpDocument(this.dni.value);
  }

  private getTpDocumentDenunciant(): string {
    return !this.enableDenunciant
      ? ''
      : this.denu_dni.value === ''
      ? 'PAS'
      : getTpDocument(this.denu_dni.value);
  }

  private descripcioRequiredValidator: ValidatorFn = (
    control: AbstractControl
  ): ValidationErrors | null => {
    const articles = control.get('article');
    const descripcio = control.get('descripcio');
    if (articles?.value !== '' || descripcio?.value !== '') return null;
    else return { descripcioRequired: true };
  };

  private articleValidator: ValidatorFn = (
    control: AbstractControl
  ): ValidationErrors | null => {
    const nomarticle = control.value;
    if (nomarticle === '') return null;
    const arti = this.articlesLlista.find(
      (article) => article.descripcio === nomarticle
    );
    if (arti) return null;
    else return { articleNotFound: true };
  };

  isDisabled(): boolean {
    return this.acta.kacta !== undefined;
  }
}
