import { HttpClientTestingModule } from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Router } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { provideMockStore } from '@ngrx/store/testing';
import { of } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { ActaDTO } from '../../model/acta.dto';
import { SancioService } from '../../services/sancio.service';
import { ActesListComponent } from './actes-list.component';

//mock lista de categorías

const acta: ActaDTO = new ActaDTO();
const actesList: ActaDTO[] = [acta];

const initialAuthState = {
  authentication: new AuthDTO(
    'admin',
    'acces_token',
    'refresh_token',
    false,
    []
  ),
  loggedIn: false,
  processing: false,
  logged: true,
  passwordChanged: false,
  error: null,
};

const initialState = { auth: initialAuthState };

class TemporalComponentForRoutes {}
describe('ActesListComponent', () => {
  let component: ActesListComponent;
  let fixture: ComponentFixture<ActesListComponent>;

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [
        HttpClientTestingModule,
        ReactiveFormsModule,
        FormsModule,
        RouterTestingModule.withRoutes([
          {
            path: 'sancio/menu',
            component: TemporalComponentForRoutes,
          },
        ]),
      ],
      declarations: [ActesListComponent],
      providers: [SancioService, provideMockStore({ initialState })],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    }).compileComponents()
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(ActesListComponent);
    //    store = TestBed.inject(MockStore);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('ActesListComponent creado correctamente', () => {
    expect(component).toBeTruthy();
  });

  it('filterByDocument funciona correctamente desde una suscripción', () => {
    const sancioService = fixture.debugElement.injector.get(SancioService);

    const spy = spyOn(sancioService, 'filterByDocument').and.returnValue(
      of(actesList)
    );
    component.document.setValue('12345678Z');
    component['filterByDocument']();
    expect(spy).toHaveBeenCalled();
    expect(component.actes).toEqual(actesList);
  });

  it('filterByNumActa funciona correctamente desde una suscripción', () => {
    const sancioService = fixture.debugElement.injector.get(SancioService);

    const spy = spyOn(sancioService, 'filterByNumActa').and.returnValue(
      of(actesList)
    );
    component.year.setValue('2022');
    component.numActa.setValue('100');
    component['filterByNumActa']();
    expect(spy).toHaveBeenCalled();
    expect(component.actes).toBe(actesList);
  });

  it('Navegamos correctamente cunado queremos mostrar la información de una sanción', () => {
    const router = TestBed.inject(Router);
    const spy = spyOn(router, 'navigateByUrl');
    acta.kacta = 1;
    component.updateActa(acta);
    expect(spy).toHaveBeenCalledWith('sancio/menu');
  });

  it('Navegamos correctamente cunado queremos añadir una nueva sanción', () => {
    const router = TestBed.inject(Router);
    const spy = spyOn(router, 'navigateByUrl');
    component.afegirActa();
    expect(spy).toHaveBeenCalledWith('sancio/menu');
  });
});
