import { HttpClientTestingModule } from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Router } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { provideMockStore } from '@ngrx/store/testing';
import { of } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { ActaDTO } from '../../model/acta.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { SancioService } from '../../services/sancio.service';
import { ImpressioComponent } from './impressio.component';

const initialAuthState = {
  authentication: new AuthDTO(
    'admin',
    'acces_token',
    'refresh_token',
    false,
    []
  ),
  loggedIn: false,
  processing: false,
  logged: true,
  passwordChanged: false,
  error: null,
};

const initialState = { auth: initialAuthState };
class TemporalComponentForRoutes {}

describe('ImpressioComponent', () => {
  let component: ImpressioComponent;
  let fixture: ComponentFixture<ImpressioComponent>;
  let menuSancioService: MenuSancioService;
  let sancioService: SancioService;
  let router: Router;

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [
        HttpClientTestingModule,
        ReactiveFormsModule,
        FormsModule,
        RouterTestingModule.withRoutes([
          {
            path: 'sancio/menu/insite',
            component: TemporalComponentForRoutes,
          },
        ]),
      ],
      declarations: [ImpressioComponent],
      providers: [provideMockStore({ initialState })],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    }).compileComponents()
  );

  beforeEach(() => {
    menuSancioService = TestBed.inject(MenuSancioService);
    sancioService = TestBed.inject(SancioService);
  });

  it('ImpressioComponent creado correctamente', () => {
    fixture = TestBed.createComponent(ImpressioComponent);
    //    store = TestBed.inject(MockStore);
    component = fixture.componentInstance;
    fixture.detectChanges();

    expect(component).toBeTruthy();
  });

  it('Se llama a la la funcion addActa del servicio SancioService al construir el componente', () => {
    let acta = new ActaDTO();
    menuSancioService.setActa(acta);

    const spy_save = spyOn(sancioService, 'addActa').and.returnValue(of(acta));

    fixture = TestBed.createComponent(ImpressioComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    expect(spy_save).toHaveBeenCalled();
    expect(component.acta).toBe(acta);
  });

  it('Se llama a la la funcion cobroEfectiu del servicio SancioService al cobrar en Efectivo', () => {
    let acta = new ActaDTO();
    menuSancioService.setActa(acta);

    fixture = TestBed.createComponent(ImpressioComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    const spy_cobro = spyOn(sancioService, 'cobroEfectiu').and.returnValue(
      of(acta)
    );
    component['cobroSancioEfectiu']();
    expect(spy_cobro).toHaveBeenCalled();
    expect(component.acta).toBe(acta);
  });
});
