import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { Router } from '@angular/router';
import { MessageService } from 'src/app/Shared/services/message.service';
import { ActaDTO } from '../../model/acta.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { SancioService } from '../../services/sancio.service';
import { TpvService } from '../../services/tpv.service';
@Component({
  selector: 'app-impressio',
  templateUrl: './impressio.component.html',
  styleUrls: ['./impressio.component.scss'],
})
export class ImpressioComponent implements OnInit {
  isSaved: boolean;
  acta: ActaDTO;

  constructor(
    private formBuilder: FormBuilder,
    private messageService: MessageService,
    private menuSancioService: MenuSancioService,
    private sancioService: SancioService,
    private router: Router,
    private tpvService: TpvService
  ) {
    this.isSaved = false;
    this.acta = this.menuSancioService.getActa();
    if (!this.acta.kacta) this.saveActa();
    else {
      this.loadFirmesIfNeeded();
      this.isSaved = true;
    }
  }

  ngOnInit(): void {}

  situacio(): string {
    let situacio = '';
    situacio += this.acta.situktipcarrer;
    situacio += ' ' + this.acta.situnomvia;
    situacio +=
      this.acta.situampliacio !== '' ? ' ' + this.acta.situampliacio : '';
    situacio += this.acta.situnro ? ' ' + this.acta.situnro : '';
    situacio += this.acta.situkm ? ' Km: ' + this.acta.situkm : '';
    situacio += this.acta.situlletra ? ' ' + this.acta.situlletra : '';
    situacio += this.acta.situportal ? ' ' + this.acta.situportal : '';
    situacio += this.acta.situbloc ? ' ' + this.acta.situbloc : '';
    situacio += this.acta.situescala ? ' ' + this.acta.situescala : '';
    situacio += this.acta.situplanta ? ' ' + this.acta.situplanta : '';
    situacio += this.acta.situporta ? ' ' + this.acta.situporta : '';
    return situacio;
  }

  data(): string {
    return new Date().toString();
    //    return data.substring(8,9)+'/'+data.substring(5,7)+'/'+data.substring(0,4)+' '+data.substring(0,4)+':'+data.substring(0,4);
  }

  //https://mvnrepository.com/artifact/com.fasterxml.jackson.datatype/jackson-datatype-jsr310/2.13.2
  saveActa(): void {
    this.sancioService.addActa(this.acta).subscribe(
      (acta: ActaDTO) => {
        this.isSaved = true;
        this.acta = acta;
      },
      (error: HttpErrorResponse) => {
        this.messageService.showHttpResponseError('impressioFeedback', error);
      }
    );
  }

  cobroSancioEfectiu(): void {
    this.sancioService.cobroEfectiu(this.acta).subscribe(
      (acta) => {
        this.acta = acta;
      },
      (error: HttpErrorResponse) => {
        this.messageService.showHttpResponseError('impressioFeedback', error);
      }
    );
  }

  cobroTargeta(): void {
    if (
      this.acta.article.importSancio &&
      this.acta.formattedData &&
      this.acta.numActa
    ) {
      this.router.navigateByUrl('sancio/menu/insite');
    } else {
      console.log('Error imprevisto');
    }
  }

  print() {
    window.print();
  }

  esPotCobrar(): boolean {
    return (
      this.acta.rebut != undefined &&
      this.acta.rebut.krebut != undefined &&
      this.acta.rebut.estat === 'P'
    );
  }

  private loadFirmesIfNeeded(): void {
    if (
      this.acta.kacta &&
      (!this.acta.firmes || this.acta.firmes.length === 0)
    ) {
      this.sancioService.getFirmes(this.acta.kacta.toString()).subscribe(
        (firmes) => {
          this.acta.firmes = [...firmes];
        },
        (error: HttpErrorResponse) => {
          this.messageService.showHttpResponseError('impressioFeedback', error);
        }
      );
    }
  }

  referencia(): string {
    if (this.acta.rebut && this.acta.rebut.barcode !== '') {
      return this.acta.rebut.barcode.substring(11, 23);
    } else return '';
  }

  identificacio(): string {
    if (this.acta.rebut && this.acta.rebut.barcode !== '') {
      return this.acta.rebut.barcode.substring(23, 33);
    } else return '';
  }
}
