import { Component, HostListener, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { MessageService } from 'src/app/Shared/services/message.service';
import { ActaDTO } from '../../model/acta.dto';
import { PayinfoDTO } from '../../model/payinfo.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { TpvService } from '../../services/tpv.service';

const msgs = [
  {
    code: 'msg1',
    description: 'Ha de rellenar los datos de la tarjeta',
  },
  {
    code: 'msg2',
    description: 'La tarjeta es obligatoria',
  },
  {
    code: 'msg3',
    description: 'La tarjeta ha de ser numérica',
  },
  {
    code: 'msg4',
    description: 'La tarjeta no puede ser negativa',
  },
  {
    code: 'msg5',
    description: 'El mes de caducidad de la tarjeta es obligatorio',
  },
  {
    code: 'msg6',
    description: 'El mes de caducidad de la tarjeta ha de ser numérico',
  },
  {
    code: 'msg7',
    description: 'El mes de caducidad de la tarjeta es incorrecto',
  },
  {
    code: 'msg8',
    description: 'El año de caducidad de la tarjeta es obligatorio',
  },
  {
    code: 'msg9',
    description: 'El año de caducidad de la tarjeta ha de ser numérico',
  },
  {
    code: 'msg10',
    description: 'El año de caducidad de la tarjeta no puede ser negativo',
  },
  {
    code: 'msg11',
    description:
      'El código de seguridad de la tarjeta no tiene la longitud correcta',
  },
  {
    code: 'msg12',
    description: 'El código de seguridad de la tarjeta ha de ser numérico',
  },
  {
    code: 'msg13',
    description: 'El código de seguridad de la tarjeta no puede ser negativo',
  },
  {
    code: 'msg14',
    description: 'El código de seguridad no es necesario para su tarjeta',
  },
  {
    code: 'msg15',
    description: 'La longitud de la tarjeta no es correcta',
  },
  {
    code: 'msg16',
    description:
      'Debe Introducir un número de tarjeta válido (sin espacios ni guiones)',
  },
  {
    code: 'msg17',
    description: 'Validación incorrecta por parte del comercio',
  },
];

declare function storeIdOper(
  event: any,
  token: any,
  errorcode: any,
  funct: any
): any;
declare function getInSiteForm(
  id: any,
  styleButton: any,
  styleBody: any,
  styleBox: any,
  styleBoxText: any,
  buttonValue: any,
  fuc: any,
  terminal: any,
  order: any,
  idioma: any,
  mostrarLogo: any
): any;

@Component({
  selector: 'app-insite',
  templateUrl: './insite.component.html',
  styleUrls: ['./insite.component.scss'],
})
export class InsiteComponent implements OnInit {
  @HostListener('window:message', ['$event'])
  onMessage(event: any) {
    if (!this.processed) {
      storeIdOper(event, 'token', 'errorCode', null);
      let errorCode = (document.getElementById('errorCode') as HTMLInputElement)
        .value;
      if (errorCode !== '')
        this.messageService.showMessage(
          'tpvFeedback',
          this.getErrorDescription(errorCode)
        );
      else {
        let orderid = (document.getElementById('token') as HTMLInputElement)
          .value;
        if (orderid !== '') {
          this.processed = true;
          this.payinfo.orderid = orderid;

          this.tpvService.prepay(this.payinfo).subscribe(
            (valinfo) => {
              this.menuSancioService.setValinfo(valinfo);
              this.router.navigateByUrl('sancio/menu/redsysval');
            },
            (error) => {
              this.messageService.showHttpResponseError('tpvFeedback', error);
            }
          );
        }
      }
    }
  }

  processed: boolean;
  acta: ActaDTO;
  payinfo: PayinfoDTO;

  getErrorDescription(code: string): string {
    let msg = msgs.find((msg) => {
      return (msg.code = code);
    });

    let descr = !msg ? code : msg.description;
    return descr;
  }

  constructor(
    private router: Router,
    private messageService: MessageService,
    private tpvService: TpvService,
    private menuSancioService: MenuSancioService
  ) {
    this.acta = menuSancioService.getActa();
    this.payinfo = new PayinfoDTO();
    this.processed = false;
    if (
      this.acta.article.importSancio &&
      this.acta.formattedData &&
      this.acta.numActa
    ) {
      this.payinfo.amount = (this.acta.article.importSancio / 2).toString();
      this.payinfo.year = this.acta.formattedData.substring(6, 10);
      this.payinfo.numacta = this.acta.numActa.toString();
    }
  }

  ngOnInit(): void {
    this.payinfo.orderinfo = this.calcOrderInfo();
    getInSiteForm(
      'card-form',
      'background-color: #3f51b5',
      '',
      '',
      '',
      'Pago sanción',
      '999008881',
      '1',
      this.payinfo.orderinfo,
      'ES',
      false
    );
  }

  calcOrderInfo(): string {
    let order = '';
    if (this.acta.formattedData && this.acta.numActa)
      /*      order =
        'SPGS' +
        this.acta.formattedData?.substring(6, 10) +
        (this.acta.numActa / 1000000).toString().substring(2) +
        Math.floor(Math.random() * 100 + 1);*/
      order = 'pedido' + Math.floor(Math.random() * 100000 + 1);
    return order;
  }

  exit(): void {
    this.router.navigateByUrl('sancio/menu/impressio');
  }
}
