import { Location } from '@angular/common';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Router } from '@angular/router';
import { Subscription } from 'rxjs';
import { ActaDTO } from '../../model/acta.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';

@Component({
  selector: 'app-menu-sancio',
  templateUrl: './menu-sancio.component.html',
  styleUrls: ['./menu-sancio.component.scss'],
})
export class MenuSancioComponent implements OnInit, OnDestroy {
  url_retorn: string;
  desti: string;
  titul: string;
  subscription: Subscription;
  current_option: string;
  default_style?: string;
  completats: string[];
  requerits: string[];
  acta: ActaDTO;
  //https://stackoverflow.com/questions/45949476/how-to-call-router-outlet-child-component-method-from-parent-comonent

  constructor(
    private router: Router,
    private menuSancioService: MenuSancioService,
    public dialog: MatDialog,
    private location: Location
  ) {
    this.desti = 'acta';
    this.current_option = this.desti;
    this.completats = ['acta'];
    this.requerits = ['acta', 'notificacions', 'situacio', 'firma'];
    this.titul = 'Alta Nueva Sanción';
    this.acta = menuSancioService.getActa();
    this.url_retorn = menuSancioService.getURL();
    if (this.acta.kacta) {
      this.titul =
        'Consulta / Reimpresión' +
        (this.acta.rebut && this.acta.rebut.estat === 'P' ? ' / Cobro' : '') +
        ' Sanción';
    }

    this.subscription = menuSancioService.validated.subscribe(
      (isValid: boolean) => {
        this.validateForm(isValid);
      }
    );
  }

  private validateForm(isValid: boolean) {
    if (isValid) {
      if (this.desti !== '') {
        if (this.default_style) {
          const currentElement = document.getElementById(this.current_option);
          const destiElement = document.getElementById(this.desti);
          if (currentElement) {
            currentElement.className =
              this.default_style +
              (this.current_option === 'impressio' ? ' saveButton' : '');
          }
          if (destiElement) {
            destiElement.className =
              this.default_style +
              ' selected' +
              (this.desti === 'impressio' ? ' saveButton' : '');
          }
          this.completar();
          this.current_option = this.desti;
        }
        this.router.navigateByUrl('sancio/menu/' + this.desti);
      }
    } else {
      this.dialog.open(DialogInvalidMessage);
    }
  }

  completar() {
    if (
      this.requerits.includes(this.desti) &&
      !this.completats.includes(this.desti)
    ) {
      this.completats.push(this.desti);
    }
  }

  ngOnInit(): void {
    this.default_style = document.getElementById('notificacions')?.className;
  }

  denuncia() {
    this.desti = 'acta';
    this.menuSancioService.change.emit();
  }

  notificacions() {
    this.desti = 'notificacions';
    this.menuSancioService.change.emit();
  }

  situacio() {
    this.desti = 'situacio';
    this.menuSancioService.change.emit();
  }

  fotos() {
    this.desti = 'fotos';
    this.menuSancioService.change.emit();
  }

  firmes() {
    this.desti = 'firma';
    this.menuSancioService.change.emit();
  }

  impressio() {
    this.desti = 'impressio';
    this.menuSancioService.change.emit();
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  isCompleted(): boolean {
    return (
      this.acta.kacta !== undefined ||
      this.completats.length === this.requerits.length
    );
  }

  back(): void {
    this.router.navigateByUrl(this.url_retorn);
  }
}

@Component({
  selector: 'dialog-invalid-message',
  templateUrl: 'invalid-message.html',
})
export class DialogInvalidMessage {}
