import { TextFieldModule } from '@angular/cdk/text-field';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { MatSlideToggleModule } from '@angular/material/slide-toggle';
import { provideMockStore } from '@ngrx/store/testing';
import { of } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { ActaDTO } from '../../model/acta.dto';
import { IdentificacioDTO } from '../../model/identificacio.dto';
import { MunicipiDTO } from '../../model/municipi.dto';
import { NotificacioDTO } from '../../model/notificacio.dto';
import { PaisDTO } from '../../model/pais.dto';
import { ProvinciaDTO } from '../../model/provincia.dto';
import { TipCarrerDTO } from '../../model/tipcarrer.dto';
import { ViaDTO } from '../../model/via.dto';
import { CadastreStorageService } from '../../services/cadastre-storage.service';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { NotificacionsComponent } from './notificacions.component';

const provincia: ProvinciaDTO = {
  kprovincia: 7,
  nom: 'ILLES BALEARS',
  municipis: [],
};

const municipi: MunicipiDTO = {
  kmunicipi: 11,
  nom: 'CALVIÀ',
};

const paisos: PaisDTO[] = [new PaisDTO(), new PaisDTO()];
const provincies: ProvinciaDTO[] = [provincia, new ProvinciaDTO()];
const tpvies: TipCarrerDTO[] = [new TipCarrerDTO(), new TipCarrerDTO()];
const municipis: MunicipiDTO[] = [municipi, new MunicipiDTO()];
const carrers: ViaDTO[] = [new ViaDTO(), new ViaDTO()];

const initialAuthState = {
  authentication: new AuthDTO(
    'admin',
    'acces_token',
    'refresh_token',
    false,
    []
  ),
  loggedIn: false,
  processing: false,
  logged: true,
  passwordChanged: false,
  error: null,
};

const initialState = { auth: initialAuthState };

describe('NotificacioComponent', () => {
  let component: NotificacionsComponent;
  let fixture: ComponentFixture<NotificacionsComponent>;
  let menuSancioService: MenuSancioService;
  let cadastreStorageService: CadastreStorageService;

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [
        HttpClientTestingModule,
        ReactiveFormsModule,
        FormsModule,
        MatAutocompleteModule,
        MatSlideToggleModule,
        TextFieldModule,
      ],
      declarations: [NotificacionsComponent],
      providers: [provideMockStore({ initialState })],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    }).compileComponents()
  );

  beforeEach(() => {
    menuSancioService = TestBed.inject(MenuSancioService);
    cadastreStorageService = TestBed.inject(CadastreStorageService);
  });

  it('NotificacioComponent creado correctamente', () => {
    fixture = TestBed.createComponent(NotificacionsComponent);
    component = fixture.componentInstance;
    expect(component).toBeTruthy();
  });

  it('Información catastral cargada correctamente al inicializar', () => {
    const spy_paisos = spyOn(
      cadastreStorageService,
      'getPaisos'
    ).and.returnValue(of(paisos));
    const spy_provincies = spyOn(
      cadastreStorageService,
      'getProvincies'
    ).and.returnValue(of(provincies));
    const spy_tpvies = spyOn(
      cadastreStorageService,
      'getTipCarrers'
    ).and.returnValue(of(tpvies));

    fixture = TestBed.createComponent(NotificacionsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    expect(spy_paisos).toHaveBeenCalled();
    expect(spy_provincies).toHaveBeenCalled();
    expect(spy_tpvies).toHaveBeenCalled();
    expect(component.paisosLlista).toBe(paisos);
    expect(component.provinciesLlista).toEqual(provincies);
    expect(component.tipCarrersLlista).toBe(tpvies);
  });

  it('Se cargan los municipios cuando cambiamos de provincia', () => {
    fixture = TestBed.createComponent(NotificacionsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
    component.provinciesAll = provincies;
    component.provincia.setValue(provincia.nom);
    const spy_municipis = spyOn(
      cadastreStorageService,
      'getMunicipis'
    ).and.returnValue(of(municipis));
    component['changeProvincia']();
    expect(spy_municipis).toHaveBeenCalled();
    expect(component.municipisLlista).toBe(municipis);
  });

  it('Se cargan las calles cuando cambiamos de municipio', () => {
    fixture = TestBed.createComponent(NotificacionsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
    component.municipisLlista = municipis;
    component.municipi.setValue('CALVIÀ');
    const spy_vies = spyOn(cadastreStorageService, 'getVies').and.returnValue(
      of(carrers)
    );
    component['changeMunicipi']();
    expect(spy_vies).toHaveBeenCalled();
    expect(component.viesLlista).toBe(carrers);
  });

  it('isValidForm devuelve que el formulario no es valido con datos incorrectos', () => {
    fixture = TestBed.createComponent(NotificacionsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
    const isValidForm: boolean = component.isValidForm();
    expect(isValidForm).toBe(false);
  });

  it('isValidForm devuelve que el formulario  es valido con datos correctos', () => {
    const valid_domnotif: NotificacioDTO = {
      nompais: 'ESPAÑA',
      nomprovincia: 'ILLES BALEARS',
      nommunicipi: 'CALVIÀ',
      tpvia: 'CL',
      nomvia: 'TEST',
      nro: 100,
      lletra: '',
      portal: '',
      bloc: '',
      cp: '07183',
      ampliacio: '',
      escala: '',
      planta: '',
      porta: '',
      habitacio: '',
    };

    const valid_denunciat = new IdentificacioDTO();
    valid_denunciat.domnotif = valid_domnotif;

    const actaValida = new ActaDTO();

    actaValida.denunciat = valid_denunciat;
    menuSancioService.setActa(actaValida);

    fixture = TestBed.createComponent(NotificacionsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    const isValidForm: boolean = component.isValidForm();
    expect(isValidForm).toBe(true);
  });
});
