import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnDestroy, OnInit } from '@angular/core';
import {
  AbstractControl,
  FormBuilder,
  FormControl,
  FormGroup,
  ValidationErrors,
  ValidatorFn,
  Validators,
} from '@angular/forms';
import { Observable, Subscription } from 'rxjs';
import { map, startWith } from 'rxjs/operators';
import { MessageService } from 'src/app/Shared/services/message.service';
import { ActaDTO } from '../../model/acta.dto';
import { MunicipiDTO } from '../../model/municipi.dto';
import { NotificacioDTO } from '../../model/notificacio.dto';
import { PaisDTO } from '../../model/pais.dto';
import { ProvinciaDTO } from '../../model/provincia.dto';
import { TipCarrerDTO } from '../../model/tipcarrer.dto';
import { ViaDTO } from '../../model/via.dto';
import { CadastreStorageService } from '../../services/cadastre-storage.service';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { SancioStorageService } from '../../services/sancio-storage.service';

@Component({
  selector: 'app-notificacions',
  templateUrl: './notificacions.component.html',
  styleUrls: ['./notificacions.component.scss'],
})
export class NotificacionsComponent implements OnInit, OnDestroy {
  acta: ActaDTO;
  notifForm: FormGroup;
  accForm: FormGroup;

  paisosLlista: PaisDTO[];
  provinciesAll: ProvinciaDTO[];
  provinciesLlista: ProvinciaDTO[];
  municipisLlista: MunicipiDTO[];
  viesLlista: ViaDTO[];
  tipCarrersLlista: TipCarrerDTO[];
  old_pais: string;
  pais: FormControl;
  filteredPaisos: Observable<PaisDTO[]>;

  old_provincia: string;
  provincia: FormControl;

  filteredProvincies: Observable<ProvinciaDTO[]>;

  old_municipi: string;
  municipi: FormControl;
  filteredMunicipis: Observable<MunicipiDTO[]>;

  tpvia: FormControl;
  filteredTpCarrers: Observable<TipCarrerDTO[]>;

  nomcarrer: FormControl;
  filteredCarrers: Observable<ViaDTO[]>;

  ampliacio: FormControl;
  nro: FormControl;
  km: FormControl;
  lletra: FormControl;
  portal: FormControl;
  bloc: FormControl;
  esca: FormControl;
  planta: FormControl;
  puer: FormControl;
  cp: FormControl;

  acc_provinciesLlista: ProvinciaDTO[];
  acc_municipisLlista: MunicipiDTO[];
  acc_viesLlista: ViaDTO[];

  acc_old_pais: string;
  acc_pais: FormControl;
  acc_filteredPaisos: Observable<PaisDTO[]>;

  acc_old_provincia: string;
  acc_provincia: FormControl;

  acc_filteredProvincies: Observable<ProvinciaDTO[]>;

  acc_old_municipi: string;
  acc_municipi: FormControl;
  acc_filteredMunicipis: Observable<MunicipiDTO[]>;

  acc_tpvia: FormControl;
  acc_filteredTpCarrers: Observable<TipCarrerDTO[]>;

  acc_nomcarrer: FormControl;
  acc_filteredCarrers: Observable<ViaDTO[]>;

  acc_ampliacio: FormControl;
  acc_nro: FormControl;
  acc_km: FormControl;
  acc_lletra: FormControl;
  acc_portal: FormControl;
  acc_bloc: FormControl;
  acc_esca: FormControl;
  acc_planta: FormControl;
  acc_puer: FormControl;
  acc_cp: FormControl;
  acc_habitacio: FormControl;
  enableAccidental: FormControl;

  subscription: Subscription;
  formValidated: boolean;

  constructor(
    private formBuilder: FormBuilder,
    private cadastreStorageService: CadastreStorageService,
    private sancioStorageService: SancioStorageService,
    private messageService: MessageService,
    private menuSancioService: MenuSancioService
  ) {
    this.formValidated = false;
    this.acta = menuSancioService.getActa();
    this.subscription = menuSancioService.change.subscribe(() => {
      menuSancioService.validated.emit(
        this.acta.kacta ? true : this.isValidForm()
      );
    });

    this.paisosLlista = [];
    this.provinciesAll = [];
    this.provinciesLlista = [];
    this.municipisLlista = [];
    this.viesLlista = [];
    this.tipCarrersLlista = [];

    const domnotif = this.acta.denunciat.domnotif;
    this.old_pais = domnotif.nompais;
    this.old_provincia = domnotif.nomprovincia;
    this.old_municipi = domnotif.nommunicipi;

    this.ampliacio = new FormControl(domnotif.ampliacio);
    this.nro = new FormControl(domnotif.nro);
    this.km = new FormControl(domnotif.km);
    this.lletra = new FormControl(domnotif.lletra);
    this.portal = new FormControl(domnotif.portal);
    this.bloc = new FormControl(domnotif.bloc);
    this.esca = new FormControl(domnotif.escala);
    this.planta = new FormControl(domnotif.planta);
    this.puer = new FormControl(domnotif.porta);
    this.cp = new FormControl(domnotif.cp, Validators.required);

    this.pais = new FormControl(domnotif.nompais, Validators.required);
    this.filteredPaisos = this.pais.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterPaisos(value))
    );

    this.provincia = new FormControl(
      domnotif.nomprovincia,
      Validators.required
    );
    this.filteredProvincies = this.provincia.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterProvincies(value))
    );

    this.municipi = new FormControl(domnotif.nommunicipi, Validators.required);
    this.filteredMunicipis = this.municipi.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterMunicipis(value))
    );

    this.nomcarrer = new FormControl(domnotif.nomvia, Validators.required);

    this.filteredCarrers = this.nomcarrer.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterCarrers(value))
    );

    this.tpvia = new FormControl();
    this.tpvia.setValue(domnotif.tpvia);

    this.filteredTpCarrers = this.tpvia.valueChanges.pipe(
      startWith(''),
      map((value: string) =>
        this.tipCarrersLlista.filter((tpvia) =>
          tpvia.ktipcarrer.toUpperCase().includes(value.toUpperCase())
        )
      )
    );
    this.notifForm = this.formBuilder.group(
      {
        nompais: this.pais,
        nomprovincia: this.provincia,
        nommunicipi: this.municipi,
        tpvia: this.tpvia,
        nomvia: this.nomcarrer,
        ampliacio: this.ampliacio,
        nro: this.nro,
        km: this.km,
        lletra: this.lletra,
        portal: this.portal,
        bloc: this.bloc,
        escala: this.esca,
        planta: this.planta,
        porta: this.puer,
        cp: this.cp,
      },
      { validators: [this.nrokmRequiredValidator] }
    );

    this.acc_provinciesLlista = [];
    this.acc_municipisLlista = [];
    this.acc_viesLlista = [];

    if (!this.acta.domacc) this.acta.domacc = new NotificacioDTO();
    const domacc = this.acta.domacc;
    this.acc_old_pais = domacc.nompais;
    this.acc_old_provincia = domacc.nomprovincia;
    this.acc_old_municipi = domacc.nommunicipi;

    this.acc_ampliacio = new FormControl(domacc.ampliacio);
    this.acc_nro = new FormControl(domacc.nro);
    this.acc_km = new FormControl(domacc.km);
    this.acc_lletra = new FormControl(domacc.lletra);
    this.acc_portal = new FormControl(domacc.portal);
    this.acc_bloc = new FormControl(domacc.bloc);
    this.acc_esca = new FormControl(domacc.escala);
    this.acc_planta = new FormControl(domacc.planta);
    this.acc_puer = new FormControl(domacc.porta);
    this.acc_cp = new FormControl(domacc.cp);
    this.acc_habitacio = new FormControl(domacc.habitacio);

    this.acc_pais = new FormControl(domacc.nompais, this.valueRequired);
    this.acc_filteredPaisos = this.acc_pais.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterPaisos(value))
    );

    this.acc_provincia = new FormControl(
      domacc.nomprovincia,
      this.valueRequired
    );
    this.acc_filteredProvincies = this.acc_provincia.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterProvinciesAcc(value))
    );

    this.acc_municipi = new FormControl(domacc.nommunicipi, this.valueRequired);
    this.acc_filteredMunicipis = this.acc_municipi.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterMunicipisAcc(value))
    );

    this.acc_nomcarrer = new FormControl(domacc.nomvia, this.valueRequired);

    this.acc_filteredCarrers = this.acc_nomcarrer.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterCarrersAcc(value))
    );

    this.acc_tpvia = new FormControl();
    this.acc_tpvia.setValue(domacc.tpvia);

    this.acc_filteredTpCarrers = this.acc_tpvia.valueChanges.pipe(
      startWith(''),
      map((value: string) =>
        this.tipCarrersLlista.filter((tpvia) =>
          tpvia.ktipcarrer.toUpperCase().includes(value.toUpperCase())
        )
      )
    );

    this.enableAccidental = new FormControl(domacc.nomvia !== '');

    this.accForm = this.formBuilder.group(
      {
        nompais: this.acc_pais,
        nomprovincia: this.acc_provincia,
        nommunicipi: this.acc_municipi,
        tpvia: this.acc_tpvia,
        nomvia: this.acc_nomcarrer,
        ampliacio: this.acc_ampliacio,
        nro: this.acc_nro,
        km: this.acc_km,
        lletra: this.acc_lletra,
        portal: this.acc_portal,
        bloc: this.acc_bloc,
        escala: this.acc_esca,
        planta: this.acc_planta,
        porta: this.acc_puer,
        cp: this.acc_cp,
        habitacio: this.acc_habitacio,
        enableAccidental: this.enableAccidental,
      },
      { validators: [this.nrokmAccRequiredValidator] }
    );

    if (this.acta.kacta) {
      this.notifForm.disable();
      this.accForm.disable();
    }

    this.initSelects();
  }

  ngOnInit(): void {}

  changePais(): void {
    if (this.pais.value.toUpperCase() === this.old_pais.toUpperCase()) return;
    let pais = this.paisosLlista.find(
      (pais) => pais.nom === this.pais.value.toUpperCase()
    );
    if (pais) {
      this.getProvincies(pais.kpais);
    } else this.provinciesLlista = [];
    this.old_pais = this.pais.value;
    this.municipisLlista = [];
    this.viesLlista = [];
    this.provincia.setValue('');
    this.municipi.setValue('');
    this.tpvia.setValue('');
    this.nomcarrer.setValue('');
  }

  changePaisAcc(): void {
    if (this.acc_pais.value.toUpperCase() === this.acc_old_pais.toUpperCase())
      return;
    let pais = this.paisosLlista.find(
      (pais) => pais.nom === this.acc_pais.value.toUpperCase()
    );
    if (pais) {
      this.getProvinciesAcc(pais.kpais);
    } else this.acc_provinciesLlista = [];
    this.acc_old_pais = this.acc_pais.value;
    this.acc_municipisLlista = [];
    this.acc_viesLlista = [];
    this.acc_provincia.setValue('');
    this.acc_municipi.setValue('');
    this.acc_tpvia.setValue('');
    this.acc_nomcarrer.setValue('');
  }

  changeProvincia(): void {
    if (this.provincia.value.toUpperCase() === this.old_provincia.toUpperCase())
      return;
    let provincia = this.provinciesAll.find(
      (provincia) => provincia.nom === this.provincia.value.toUpperCase()
    );
    if (provincia) {
      this.loadMunicipis(provincia.kprovincia);
    } else this.municipisLlista = [];
    this.viesLlista = [];
    this.municipi.setValue('');
    this.tpvia.setValue('');
    this.nomcarrer.setValue('');
    this.old_provincia = this.provincia.value;
  }

  changeProvinciaAcc(): void {
    if (
      this.acc_provincia.value.toUpperCase() ===
      this.acc_old_provincia.toUpperCase()
    )
      return;
    let provincia = this.provinciesAll.find(
      (provincia) => provincia.nom === this.acc_provincia.value.toUpperCase()
    );
    if (provincia) {
      this.loadMunicipisAcc(provincia.kprovincia);
    } else this.acc_municipisLlista = [];
    this.acc_viesLlista = [];
    this.acc_municipi.setValue('');
    this.acc_tpvia.setValue('');
    this.acc_nomcarrer.setValue('');
    this.acc_old_provincia = this.acc_provincia.value;
  }

  changeMunicipi(): void {
    if (this.municipi.value.toUpperCase() === this.old_municipi.toUpperCase())
      return;
    let municipi = this.municipisLlista.find(
      (municipi) => municipi.nom === this.municipi.value.toUpperCase()
    );
    if (municipi) {
      this.loadVies(municipi.kmunicipi);
    } else this.viesLlista = [];
    this.tpvia.setValue('');
    this.nomcarrer.setValue('');
    this.old_municipi = this.municipi.value;
  }

  changeMunicipiAcc(): void {
    if (
      this.acc_municipi.value.toUpperCase() ===
      this.acc_old_municipi.toUpperCase()
    )
      return;
    let municipi = this.acc_municipisLlista.find(
      (municipi) => municipi.nom === this.acc_municipi.value.toUpperCase()
    );
    if (municipi) {
      this.loadViesAcc(municipi.kmunicipi);
    } else this.acc_viesLlista = [];
    this.acc_tpvia.setValue('');
    this.acc_nomcarrer.setValue('');
    this.acc_old_municipi = this.acc_municipi.value;
  }

  changeVia(): void {
    let via = this.viesLlista.find(
      (carrer) => carrer.nomvia === this.nomcarrer.value.toUpperCase()
    );
    if (via) {
      this.tpvia.setValue(via.ktipcarrer);
    }
  }

  changeViaAcc(): void {
    let via = this.acc_viesLlista.find(
      (carrer) => carrer.nomvia === this.acc_nomcarrer.value.toUpperCase()
    );
    if (via) {
      this.acc_tpvia.setValue(via.ktipcarrer);
    }
  }

  private initSelects() {
    this.loadPaisos();
    this.loadProvincies();
    this.loadTpVies();
    let domnotif = this.acta.denunciat.domnotif;
    if (domnotif.kprovincia) {
      this.loadMunicipis(domnotif.kprovincia);
    }
    if (domnotif.kmunicipi) {
      this.loadVies(domnotif.kmunicipi);
    }

    let domacc = this.acta.domacc;
    if (domacc.kprovincia) {
      this.loadMunicipisAcc(domacc.kprovincia);
    }
    if (domacc.kmunicipi) {
      this.loadViesAcc(domacc.kmunicipi);
    }
  }

  private loadPaisos(): void {
    this.cadastreStorageService.getPaisos(false).subscribe(
      (paisos: PaisDTO[]) => {
        this.paisosLlista = paisos;
        if (this.acta.denunciat.domnotif.nompais) {
          this.pais.setValue(this.acta.denunciat.domnotif.nompais);
        }
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError(
          'notificacionsFeedback',
          error
        )
    );
  }

  private loadProvincies(): void {
    this.cadastreStorageService.getProvincies(false).subscribe(
      (provincies: ProvinciaDTO[]) => {
        this.provinciesAll = provincies;
        let domnotif = this.acta.denunciat.domnotif;
        const kpais = domnotif.kpais;
        this.getProvincies(kpais);
        const kpais_acc = domnotif.kpais;
        this.getProvinciesAcc(kpais_acc);
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError(
          'notificacionsFeedback',
          error
        )
    );
  }

  private loadMunicipis(kprovicia: number): void {
    this.cadastreStorageService
      .getMunicipis(false, kprovicia.toString())
      .subscribe(
        (municipis: MunicipiDTO[]) => {
          this.municipisLlista = municipis;
        },
        (error: HttpErrorResponse) =>
          this.messageService.showHttpResponseError(
            'notificacionsFeedback',
            error
          )
      );
  }

  private loadMunicipisAcc(kprovicia: number): void {
    this.cadastreStorageService
      .getMunicipis(false, kprovicia.toString())
      .subscribe(
        (municipis: MunicipiDTO[]) => {
          this.acc_municipisLlista = municipis;
        },
        (error: HttpErrorResponse) =>
          this.messageService.showHttpResponseError(
            'notificacionsFeedback',
            error
          )
      );
  }

  private loadVies(kmunicipi: number): void {
    this.cadastreStorageService.getVies(false, kmunicipi.toString()).subscribe(
      (vies: ViaDTO[]) => {
        this.viesLlista = vies;
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError(
          'notificacionsFeedback',
          error
        )
    );
  }

  private loadViesAcc(kmunicipi: number): void {
    this.cadastreStorageService.getVies(false, kmunicipi.toString()).subscribe(
      (vies: ViaDTO[]) => {
        this.acc_viesLlista = vies;
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError(
          'notificacionsFeedback',
          error
        )
    );
  }

  private loadTpVies(): void {
    this.cadastreStorageService.getTipCarrers(false).subscribe(
      (tipCarrers: TipCarrerDTO[]) => {
        this.tipCarrersLlista = tipCarrers;
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError(
          'notificacionsFeedback',
          error
        )
    );
  }

  getProvincies(kpais?: number) {
    if (kpais && kpais === 108) {
      this.provinciesLlista = [...this.provinciesAll];
    } else {
      this.provinciesLlista = [];
    }
  }

  getProvinciesAcc(kpais?: number) {
    if (kpais && kpais === 108) {
      this.acc_provinciesLlista = [...this.provinciesAll];
    } else {
      this.acc_provinciesLlista = [];
    }
  }

  private filterPaisos(value: string): PaisDTO[] {
    const paisosTmp: PaisDTO[] = this.paisosLlista.filter((pais) =>
      pais.nom.toUpperCase().includes(value.toUpperCase())
    );
    return paisosTmp;
  }

  private filterProvincies(value: string): ProvinciaDTO[] {
    const provTmp: ProvinciaDTO[] = this.provinciesLlista.filter((prov) =>
      prov.nom.toUpperCase().includes(value.toUpperCase())
    );
    return provTmp;
  }

  private filterProvinciesAcc(value: string): ProvinciaDTO[] {
    const provTmp: ProvinciaDTO[] = this.acc_provinciesLlista.filter((prov) =>
      prov.nom.toUpperCase().includes(value.toUpperCase())
    );
    return provTmp;
  }

  private filterMunicipis(value: string): MunicipiDTO[] {
    const munTmp: MunicipiDTO[] = this.municipisLlista.filter((mun) =>
      mun.nom.toUpperCase().includes(value.toUpperCase())
    );
    return munTmp;
  }

  private filterMunicipisAcc(value: string): MunicipiDTO[] {
    const munTmp: MunicipiDTO[] = this.acc_municipisLlista.filter((mun) =>
      mun.nom.toUpperCase().includes(value.toUpperCase())
    );
    return munTmp;
  }

  private filterCarrers(value: string): ViaDTO[] {
    const viaTmp: ViaDTO[] = this.viesLlista.filter((via) =>
      via.nomvia.toUpperCase().includes(value.toUpperCase())
    );
    return viaTmp;
  }

  private filterCarrersAcc(value: string): ViaDTO[] {
    const viaTmp: ViaDTO[] = this.acc_viesLlista.filter((via) =>
      via.nomvia.toUpperCase().includes(value.toUpperCase())
    );
    return viaTmp;
  }

  private getKPais(): number | null {
    const pais = this.paisosLlista.filter(
      (pais) => pais.nom === this.pais.value.toUpperCase()
    )[0];
    return pais ? pais.kpais : null;
  }

  private getKPaisAcc(): number | null {
    const pais = this.paisosLlista.filter(
      (pais) => pais.nom === this.acc_pais.value.toUpperCase()
    )[0];
    return pais ? pais.kpais : null;
  }

  private getKProv(): number | null {
    const prov = this.provinciesAll.filter(
      (prov) => prov.nom === this.provincia.value.toUpperCase()
    )[0];
    return prov ? prov.kprovincia : null;
  }

  private getKProvAcc(): number | null {
    const prov = this.provinciesAll.filter(
      (prov) => prov.nom === this.acc_provincia.value.toUpperCase()
    )[0];
    return prov ? prov.kprovincia : null;
  }

  private getKMun(): number | null {
    const mun = this.municipisLlista.filter(
      (mun) => mun.nom === this.municipi.value.toUpperCase()
    )[0];
    return mun ? mun.kmunicipi : null;
  }

  private getKMunAcc(): number | null {
    const mun = this.acc_municipisLlista.filter(
      (mun) => mun.nom === this.acc_municipi.value.toUpperCase()
    )[0];
    return mun ? mun.kmunicipi : null;
  }

  private getKVia(): number | null {
    const via = this.viesLlista.filter(
      (via) => via.nomvia === this.nomcarrer.value.toUpperCase()
    )[0];
    return via ? via.kvia : null;
  }

  private getKViaAcc(): number | null {
    const via = this.acc_viesLlista.filter(
      (via) => via.nomvia === this.acc_nomcarrer.value.toUpperCase()
    )[0];
    return via ? via.kvia : null;
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  public isValidForm(): boolean {
    this.notifForm.markAllAsTouched();
    this.accForm.markAllAsTouched();
    this.formValidated = true;
    if (!this.notifForm.invalid && !this.accForm.invalid) {
      this.acta = {
        ...this.acta,
        denunciat: {
          ...this.acta.denunciat,
          domnotif: {
            ...this.acta.denunciat.domnotif,
            ...this.notifForm.value,
            kpais: this.getKPais(),
            kprovincia: this.getKProv(),
            kmunicipi: this.getKMun(),
            kvia: this.getKVia(),
          },
        },
        domacc: this.enableAccidental.value
          ? {
              ...this.accForm.value,
              kpais: this.getKPaisAcc(),
              kprovincia: this.getKProvAcc(),
              kmunicipi: this.getKMunAcc(),
              kvia: this.getKViaAcc(),
            }
          : null,
      };
      this.menuSancioService.setActa(this.acta);
      //      this.store.dispatch(updateActa({ acta: this.acta }));
    }
    return !this.notifForm.invalid && !this.accForm.invalid;
  }

  private nrokmRequiredValidator: ValidatorFn = (
    control: AbstractControl
  ): ValidationErrors | null => {
    const nro = control.get('nro');
    const km = control.get('km');
    if (
      (nro && nro.value != null && nro.value !== '') ||
      (km && km.value != null && km?.value !== '')
    )
      return null;
    else return { nrokmRequired: true };
  };

  private nrokmAccRequiredValidator: ValidatorFn = (
    control: AbstractControl
  ): ValidationErrors | null => {
    const nro = control.get('nro');
    const km = control.get('km');
    if (
      !this.hasAccidental() ||
      (nro && nro.value != null && nro.value !== '') ||
      (km && km.value != null && km?.value !== '')
    )
      return null;
    else return { nrokmRequired: true };
  };

  private valueRequired: ValidatorFn = (
    control: AbstractControl
  ): ValidationErrors | null => {
    if (!this.hasAccidental() || control.value !== '') return null;
    else return { required: true };
  };

  hasAccidental(): boolean {
    return this.enableAccidental && this.enableAccidental.value;
  }

  cleanAllAccidental(): void {
    this.acta.domacc = new NotificacioDTO();
    this.acc_pais.setValue(this.acta.domacc.nompais);
    this.acc_provincia.setValue('');
    this.acc_municipi.setValue('');
    this.acc_tpvia.setValue('');
    this.acc_nomcarrer.setValue('');
    this.acc_ampliacio.setValue('');
    this.acc_nro.setValue(null);
    this.acc_km.setValue(null);
    this.acc_lletra.setValue('');
    this.acc_portal.setValue('');
    this.acc_bloc.setValue('');
    this.acc_esca.setValue('');
    this.acc_planta.setValue('');
    this.acc_puer.setValue('');
    this.acc_cp.setValue('');
    this.acc_habitacio.setValue('');
    this.acc_old_pais = this.acta.domacc.nompais;
    this.acc_old_provincia = '';
    this.acc_old_municipi = '';
    this.getProvinciesAcc(this.acta.domacc.kpais);
  }
}
