import { HttpClientTestingModule } from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Router } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { provideMockStore } from '@ngrx/store/testing';
import { of } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { IdentificacioDTO } from '../../model/identificacio.dto';
import { SancionatDTO } from '../../model/sancionat.dto';
import { PersonaService } from '../../services/persona.service';
import { PersonesListComponent } from './persones-list.component';

//mock lista de categorías

const sancionat: SancionatDTO = new SancionatDTO();
const sancionatsList: SancionatDTO[] = [sancionat];
const persona: IdentificacioDTO = new IdentificacioDTO();

const initialAuthState = {
  authentication: new AuthDTO(
    'admin',
    'acces_token',
    'refresh_token',
    false,
    []
  ),
  loggedIn: false,
  processing: false,
  logged: true,
  passwordChanged: false,
  error: null,
};

const initialState = { auth: initialAuthState };

class TemporalComponentForRoutes {}
describe('PersonesListComponent', () => {
  let component: PersonesListComponent;
  let fixture: ComponentFixture<PersonesListComponent>;

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [
        HttpClientTestingModule,
        ReactiveFormsModule,
        FormsModule,
        RouterTestingModule.withRoutes([
          {
            path: 'sancio/persona-info/:kpersona',
            component: TemporalComponentForRoutes,
          },
          {
            path: 'sancio/menu',
            component: TemporalComponentForRoutes,
          },
        ]),
      ],
      declarations: [PersonesListComponent],
      providers: [PersonaService, provideMockStore({ initialState })],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    }).compileComponents()
  );

  beforeEach(() => {
    fixture = TestBed.createComponent(PersonesListComponent);
    //    store = TestBed.inject(MockStore);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('PersonesListComponent creado correctamente', () => {
    expect(component).toBeTruthy();
  });

  it('filterByDocument funciona correctamente desde una suscripción', () => {
    const personaService = fixture.debugElement.injector.get(PersonaService);

    const spy = spyOn(
      personaService,
      'findSancionatByDocument'
    ).and.returnValue(of(sancionat));
    component.document.setValue('12345678Z');
    component['filterByDocument']();
    expect(spy).toHaveBeenCalled();
    expect(component.sancionats).toEqual(sancionatsList);
  });

  it('filterByNom funciona correctamente desde una suscripción', () => {
    const personaService = fixture.debugElement.injector.get(PersonaService);

    const spy = spyOn(personaService, 'findSancionatByNom').and.returnValue(
      of(sancionatsList)
    );
    component.nom.setValue('TEST');
    component['filterByNom']();
    expect(spy).toHaveBeenCalled();
    expect(component.sancionats).toBe(sancionatsList);
  });

  it('Navegamos correctamente cunado queremos mostrar la información de un sancionado', () => {
    const router = TestBed.inject(Router);
    const spy = spyOn(router, 'navigateByUrl');
    sancionat.kpersona = 1;
    component.showInfo(sancionat);
    expect(spy).toHaveBeenCalledWith(
      'sancio/persona-info/' + sancionat.kpersona
    );
  });

  it('Navegamos correctamente cunado queremos añadir una acta desde la información de un sancionado', () => {
    const personaService = fixture.debugElement.injector.get(PersonaService);
    const spy = spyOn(personaService, 'findById').and.returnValue(of(persona));
    const router = TestBed.inject(Router);
    const spy_router = spyOn(router, 'navigateByUrl');
    component.afegirActa(sancionat);
    expect(spy).toHaveBeenCalled();
    expect(spy_router).toHaveBeenCalledWith('sancio/menu');
  });
});
