import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnDestroy, OnInit } from '@angular/core';
import {
  AbstractControl,
  FormBuilder,
  FormControl,
  FormGroup,
  ValidationErrors,
  ValidatorFn,
  Validators,
} from '@angular/forms';
import { Observable, Subscription } from 'rxjs';
import { map, startWith } from 'rxjs/operators';
import { apiconfig } from 'src/app/Shared/model/apitool';
import { MessageService } from 'src/app/Shared/services/message.service';
import { ActaDTO } from '../../model/acta.dto';
import { TipCarrerDTO } from '../../model/tipcarrer.dto';
import { ViaDTO } from '../../model/via.dto';
import { CadastreStorageService } from '../../services/cadastre-storage.service';
import { MenuSancioService } from '../../services/menu-sancions.service';

@Component({
  selector: 'app-situacio',
  templateUrl: './situacio.component.html',
  styleUrls: ['./situacio.component.scss'],
})
export class SituacioComponent implements OnInit, OnDestroy {
  acta: ActaDTO;
  situForm: FormGroup;

  viesLlista: ViaDTO[];
  tipCarrersLlista: TipCarrerDTO[];

  tpvia: FormControl;
  filteredTpCarrers: Observable<TipCarrerDTO[]>;

  nomcarrer: FormControl;
  filteredCarrers: Observable<ViaDTO[]>;

  ampliacio: FormControl;
  nro: FormControl;
  km: FormControl;
  lletra: FormControl;
  portal: FormControl;
  bloc: FormControl;
  esca: FormControl;
  planta: FormControl;
  puer: FormControl;

  subscription: Subscription;
  formValidated: boolean;

  showMap: boolean;
  //Por defecto coordenadas policia local de Calvià
  lat = 39.528142558211435;
  lng = 2.5079512865173217;

  markers: google.maps.Marker[] = [];

  geocoder = new google.maps.Geocoder();
  infowindow = new google.maps.InfoWindow();
  map: google.maps.Map | null = null;

  //39.525302113135616, 2.485438250406104
  constructor(
    private formBuilder: FormBuilder,
    private cadastreStorageService: CadastreStorageService,
    private messageService: MessageService,
    private menuSancioService: MenuSancioService
  ) {
    this.showMap = false;
    this.formValidated = false;
    this.acta = menuSancioService.getActa();
    this.subscription = menuSancioService.change.subscribe(() => {
      menuSancioService.validated.emit(
        this.acta.kacta ? true : this.isValidForm()
      );
    });

    this.viesLlista = [];
    this.tipCarrersLlista = [];

    this.ampliacio = new FormControl(this.acta.situampliacio);
    this.nro = new FormControl(this.acta.situnro);
    this.km = new FormControl(this.acta.situkm);
    this.lletra = new FormControl(this.acta.situlletra);
    this.portal = new FormControl(this.acta.situportal);
    this.bloc = new FormControl(this.acta.situbloc);
    this.esca = new FormControl(this.acta.situescala);
    this.planta = new FormControl(this.acta.situplanta);
    this.puer = new FormControl(this.acta.situporta);

    this.nomcarrer = new FormControl(this.acta.situnomvia, Validators.required);

    this.filteredCarrers = this.nomcarrer.valueChanges.pipe(
      startWith(''),
      map((value: string) => this.filterCarrers(value))
    );

    this.tpvia = new FormControl();
    this.tpvia.setValue(this.acta.situktipcarrer);

    this.filteredTpCarrers = this.tpvia.valueChanges.pipe(
      startWith(''),
      map((value: string) =>
        this.tipCarrersLlista.filter((tpvia) =>
          tpvia.ktipcarrer.toUpperCase().includes(value.toUpperCase())
        )
      )
    );
    this.situForm = this.formBuilder.group(
      {
        situktipcarrer: this.tpvia,
        situnomvia: this.nomcarrer,
        situampliacio: this.ampliacio,
        situnro: this.nro,
        situkm: this.km,
        situlletra: this.lletra,
        situportal: this.portal,
        situbloc: this.bloc,
        situescala: this.esca,
        situplanta: this.planta,
        situporta: this.puer,
      },
      { validators: [this.nrokmRequiredValidator] }
    );

    if (this.acta.kacta) {
      this.situForm.disable();
    }

    this.initSelects();
  }

  ngOnInit(): void {
    //https://developers.google.com/maps/documentation/geocoding
    //https://developers.google.com/maps/documentation/javascript/geocoding#ReverseGeocoding
    //https://developers.google.com/maps/documentation/javascript/examples/geocoding-reverse
    this.getPosition()
      .then((pos) => {
        this.lat = pos.lat;
        this.lng = pos.lng;
      })
      .finally(() => {
        this.initMap();
      });
    this.initMap();
  }

  private initMap(): void {
    if (this.mapsEnabled() && document.getElementById('map')) {
      let mapDOM: HTMLElement = document.getElementById('map') as HTMLElement;

      const loc = new google.maps.LatLng(this.lat, this.lng);
      this.map = new google.maps.Map(mapDOM, {
        center: loc,
        zoom: 4,
      });

      // This event listener will call addMarker() when the map is clicked.
      // https://developers.google.com/maps/documentation/javascript/examples/marker-remove#maps_marker_remove-javascript

      this.map.addListener('click', (event) => {
        this.addMarker(event.latLng);
      });
    }
  }

  addMarker(position: google.maps.LatLng) {
    this.markers.forEach((marker) => {
      marker.setMap(null);
    });
    this.markers = [];
    this.lat = position.lat();
    this.lng = position.lng();
    if (this.map) {
      this.geocodeLatLng(
        this.geocoder,
        this.map,
        this.infowindow,
        this.markers,
        this.tpvia,
        this.nomcarrer,
        this.nro
      );
    }
  }

  geocodeLatLng(
    geocoder: google.maps.Geocoder,
    map: google.maps.Map,
    infowindow: google.maps.InfoWindow,
    markers: google.maps.Marker[],
    tpvia: FormControl,
    nomcarrer: FormControl,
    nro: FormControl
  ) {
    const latlng = {
      lat: this.lat,
      lng: this.lng,
    };
    //https://gist.github.com/SamSamskies/6033264
    geocoder.geocode({ location: latlng }, function (results, status) {
      if (results[0]) {
        markers.push(
          new google.maps.Marker({
            position: latlng,
            map: map,
          })
        );
        const formatted_address = results[0].formatted_address;
        infowindow.setContent(formatted_address);
        infowindow.open(map, markers[0]);

        map.setZoom(18);
        map.panTo(latlng);

        tpvia.setValue('CL');
        const posini = formatted_address.indexOf(' ') + 1;
        const posfi = formatted_address.indexOf(',', posini);
        nomcarrer.setValue(
          formatted_address.substring(posini, posfi).toUpperCase()
        );
        const posfinro = formatted_address.indexOf(',', posfi + 1);
        nro.setValue(
          parseInt(formatted_address.substring(posfi + 1, posfinro))
        );
      } else {
        window.alert('No results found');
      }
    });
  }

  changeVia(): void {
    let via = this.viesLlista.find(
      (carrer) => carrer.nomvia === this.nomcarrer.value.toUpperCase()
    );
    if (via) {
      this.tpvia.setValue(via.ktipcarrer);
    }
  }

  private initSelects() {
    this.loadTpVies();
    this.loadVies(apiconfig.municipi_local);
  }

  private loadVies(kmunicipi: number): void {
    this.cadastreStorageService.getVies(false, kmunicipi.toString()).subscribe(
      (vies: ViaDTO[]) => {
        this.viesLlista = vies;
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError('situacioFeedback', error)
    );
  }

  private loadTpVies(): void {
    this.cadastreStorageService.getTipCarrers(false).subscribe(
      (tipCarrers: TipCarrerDTO[]) => {
        this.tipCarrersLlista = tipCarrers;
      },
      (error: HttpErrorResponse) =>
        this.messageService.showHttpResponseError('situacioFeedback', error)
    );
  }

  private filterCarrers(value: string): ViaDTO[] {
    const viaTmp: ViaDTO[] = this.viesLlista.filter((via) =>
      via.nomvia.toUpperCase().includes(value.toUpperCase())
    );
    return viaTmp;
  }

  private getKVia(): number | null {
    const via = this.viesLlista.filter(
      (via) => via.nomvia === this.nomcarrer.value.toUpperCase()
    )[0];
    return via ? via.kvia : null;
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  public isValidForm(): boolean {
    this.situForm.markAllAsTouched();
    this.formValidated = true;
    if (!this.situForm.invalid) {
      this.acta = {
        ...this.acta,
        ...this.situForm.value,
        situkvia: this.getKVia(),
      };
      this.menuSancioService.setActa(this.acta);
      //      this.store.dispatch(updateActa({ acta: this.acta }));
    }
    return !this.situForm.invalid;
  }

  private nrokmRequiredValidator: ValidatorFn = (
    control: AbstractControl
  ): ValidationErrors | null => {
    const nro = control.get('situnro');
    const km = control.get('situkm');
    if (
      (nro && nro.value != null && nro.value !== '') ||
      (km && km.value != null && km?.value !== '')
    )
      return null;
    else return { nrokmRequired: true };
  };

  toggleMaps() {
    this.showMap = !this.showMap;
    if (this.showMap && this.map) {
      this.geocodeLatLng(
        this.geocoder,
        this.map,
        this.infowindow,
        this.markers,
        this.tpvia,
        this.nomcarrer,
        this.nro
      );
    }
  }

  mapsEnabled(): boolean {
    return !this.acta.kacta && apiconfig.enableMaps;
  }

  getPosition(): Promise<any> {
    return new Promise((resolve, reject) => {
      navigator.geolocation.getCurrentPosition(
        (resp) => {
          resolve({ lng: resp.coords.longitude, lat: resp.coords.latitude });
        },
        (err) => {
          reject(err);
        }
      );
    });
  }
}
