import { HttpErrorResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { PaisDTO } from 'src/app/Modules/sancio/model/pais.dto';
import { ProvinciaDTO } from 'src/app/Modules/sancio/model/provincia.dto';
import { TipCarrerDTO } from 'src/app/Modules/sancio/model/tipcarrer.dto';
import { CadastreService } from 'src/app/Modules/sancio/services/cadastre.service';
import { LocalStorageService } from 'src/app/Shared/services/local-storage.service';
import { MunicipiDTO } from '../model/municipi.dto';
import { ViaDTO } from '../model/via.dto';

@Injectable({
  providedIn: 'root',
})
export class CadastreStorageService {
  constructor(
    private cadastreService: CadastreService,
    private localStorageService: LocalStorageService
  ) {}

  getProvincies(forceReload: boolean): Observable<ProvinciaDTO[]> {
    return new Observable((observer) => {
      let localprovincies: ProvinciaDTO[] = this.localStorageService.get(
        'SancionsCalviaProvincies'
      );
      if (forceReload || !localprovincies) {
        this.cadastreService
          .getProvincies()
          .subscribe((provincies: ProvinciaDTO[]) => {
            localprovincies = provincies;
            this.localStorageService.set(
              'SancionsCalviaProvincies',
              localprovincies
            );
            observer.next(localprovincies);
          }, (error: HttpErrorResponse)=>observer.error(error));
      } else {
        observer.next(localprovincies);
      }
    });
  }

  getMunicipis(
    forceReload: boolean,
    kprovincia: string
  ): Observable<MunicipiDTO[]> {
    return new Observable((observer) => {
      let localmunicipis: MunicipiDTO[] = this.localStorageService.get(
        'SancionsCalviaMunicipis_' + kprovincia
      );
      if (forceReload || !localmunicipis) {
        this.cadastreService
          .getMunicipis(kprovincia)
          .subscribe((municipis: MunicipiDTO[]) => {
            localmunicipis = municipis;
            this.localStorageService.set(
              'SancionsCalviaMunicipis_' + kprovincia,
              localmunicipis
            );
            observer.next(localmunicipis);
          }, (error: HttpErrorResponse)=>observer.error(error));
      } else {
        observer.next(localmunicipis);
      }
    });
  }

  getVies(forceReload: boolean, kmunicipi: string): Observable<ViaDTO[]> {
    return new Observable((observer) => {
      let localvies: ViaDTO[] = this.localStorageService.get(
        'SancionsCalviaVies_' + kmunicipi
      );
      if (forceReload || !localvies) {
        this.cadastreService.getVies(kmunicipi).subscribe((vies: ViaDTO[]) => {
          localvies = vies;
          this.localStorageService.set(
            'SancionsCalviaVies_' + kmunicipi,
            localvies
          );
          observer.next(localvies);
        }, (error: HttpErrorResponse)=>observer.error(error));
      } else {
        observer.next(localvies);
      }
    });
  }

  getPaisos(forceReload: boolean): Observable<PaisDTO[]> {
    return new Observable((observer) => {
      let localpaisos: PaisDTO[] = this.localStorageService.get(
        'SancionsCalviaPaisos'
      );
      if (forceReload || !localpaisos) {
        this.cadastreService.getPaisos().subscribe((paisos: PaisDTO[]) => {
          localpaisos = paisos;
          this.localStorageService.set('SancionsCalviaPaisos', localpaisos);
          observer.next(localpaisos);
        }, (error: HttpErrorResponse)=>observer.error(error));
      } else {
        observer.next(localpaisos);
      }
    });
  }

  getTipCarrers(forceReload: boolean): Observable<TipCarrerDTO[]> {
    return new Observable((observer) => {
      let localtipcarrers: TipCarrerDTO[] = this.localStorageService.get(
        'SancionsCalviaTipCarrers'
      );
      if (forceReload || !localtipcarrers) {
        this.cadastreService
          .getTipCarrers()
          .subscribe((tipcarrers: TipCarrerDTO[]) => {
            localtipcarrers = tipcarrers;
            this.localStorageService.set(
              'SancionsCalviaTipCarrers',
              localtipcarrers
            );
            observer.next(localtipcarrers);
          }, (error: HttpErrorResponse)=>observer.error(error));
      } else {
        observer.next(localtipcarrers);
      }
    });
  }
}
