import {
  HttpClientTestingModule,
  HttpTestingController,
} from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { TestBed } from '@angular/core/testing';
import { apiconfig } from 'src/app/Shared/model/apitool';
import { MunicipiDTO } from '../model/municipi.dto';
import { PaisDTO } from '../model/pais.dto';
import { ProvinciaDTO } from '../model/provincia.dto';
import { ViaDTO } from '../model/via.dto';
import { CadastreService } from './cadastre.service';

const paisos: PaisDTO[] = [new PaisDTO(), new PaisDTO()];
const provincies: ProvinciaDTO[] = [new ProvinciaDTO(), new ProvinciaDTO()];
const municipis: MunicipiDTO[] = [new MunicipiDTO(), new MunicipiDTO()];
const vies: ViaDTO[] = [new ViaDTO(), new ViaDTO()];

describe('CadastreService', () => {
  let service: CadastreService;
  let httpMock: HttpTestingController;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [CadastreService],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    });
  });

  beforeEach(() => {
    service = TestBed.inject(CadastreService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  afterEach(() => {
    httpMock.verify();
  });

  it('Cadastre Service creado correctamente', () => {
    expect(service).toBeTruthy();
  });

  it('getPaisos retorna una lista de paises con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    service.getPaisos().subscribe((resp: PaisDTO[]) => {
      expect(resp).toEqual(paisos);
    });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(apiconfig.apiurl + 'Cadastre/paisos');
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(paisos);
  });
  it('getProvincies retorna una lista de provincias con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    service.getProvincies().subscribe((resp: ProvinciaDTO[]) => {
      expect(resp).toEqual(provincies);
    });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(apiconfig.apiurl + 'Cadastre/provincies');
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(provincies);
  });
  it('getMunicipis retorna una lista de municipios con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    service.getMunicipis('7').subscribe((resp: MunicipiDTO[]) => {
      expect(resp).toEqual(municipis);
    });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(
      apiconfig.apiurl + 'Cadastre/municipis?kprovincia=7'
    );
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(municipis);
  });
  it('getVies retorna una lista de calles con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    service.getVies('11').subscribe((resp: ViaDTO[]) => {
      expect(resp).toEqual(vies);
    });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(
      apiconfig.apiurl + 'Cadastre/vies?kmunicipi=11'
    );
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(vies);
  });
});
