import {
  HttpClientTestingModule,
  HttpTestingController,
} from '@angular/common/http/testing';
import { CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA } from '@angular/core';
import { TestBed } from '@angular/core/testing';
import { apiconfig } from 'src/app/Shared/model/apitool';
import { IdentificacioDTO } from '../model/identificacio.dto';
import { SancionatDTO } from '../model/sancionat.dto';
import { PersonaService } from './persona.service';

const persona: IdentificacioDTO = new IdentificacioDTO();
const sancionat: SancionatDTO = new SancionatDTO();

const sancionats: SancionatDTO[] = [new SancionatDTO(), new SancionatDTO()];

describe('PersonaService', () => {
  let service: PersonaService;
  let httpMock: HttpTestingController;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [PersonaService],
      schemas: [CUSTOM_ELEMENTS_SCHEMA, NO_ERRORS_SCHEMA],
    });
  });

  beforeEach(() => {
    service = TestBed.inject(PersonaService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  afterEach(() => {
    httpMock.verify();
  });

  it('Persona Service creado correctamente', () => {
    expect(service).toBeTruthy();
  });

  it('findById retorna una persona con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    persona.kpersona = 1;
    service.findById(persona.kpersona).subscribe((resp: IdentificacioDTO) => {
      expect(resp).toEqual(persona);
    });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(
      apiconfig.apiurl + 'persona/findById/' + persona.kpersona
    );
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(persona);
  });

  it('findByNif retorna una persona con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    persona.kpersona = 1;
    persona.document = '12345678Z';
    service
      .findByDocument(persona.document)
      .subscribe((resp: IdentificacioDTO) => {
        expect(resp).toEqual(persona);
      });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(
      apiconfig.apiurl + 'persona/findByDocument/' + persona.document
    );
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(persona);
  });

  it('findByPassaport retorna una persona con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    persona.kpersona = 1;
    persona.passaport = '11111111';
    service
      .findByPassaport(persona.passaport)
      .subscribe((resp: IdentificacioDTO) => {
        expect(resp).toEqual(persona);
      });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(
      apiconfig.apiurl + 'persona/findByPassaport/' + persona.passaport
    );
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(persona);
  });

  it('findSancionatByDocument retorna un sancionado con llamada de tipo GET', () => {
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    sancionat.kpersona = 1;
    sancionat.document = '12345678Z';
    service
      .findSancionatByDocument(sancionat.document)
      .subscribe((resp: SancionatDTO) => {
        expect(resp).toEqual(sancionat);
      });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(
      apiconfig.apiurl + 'persona/findSancionatByDocument/' + sancionat.document
    );
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(sancionat);
  });

  it('findSancionatByNom retorna una lista de sancionados con llamada de tipo GET', () => {
    const name = 'PEDRO';
    //Nos suscribimos a la llamada al servicio, el resultado debería ser igual al mock
    service.findSancionatByNom(name).subscribe((resp: SancionatDTO[]) => {
      expect(resp).toEqual(sancionats);
    });
    //Definimos la petición mock a la url correcta
    const req = httpMock.expectOne(
      apiconfig.apiurl + 'persona/filterByName?nom=' + name
    );
    //Verificamos que el método sea del tipo GET
    expect(req.request.method).toBe('GET');

    //Lanzamos la simulación a la petición reemplazando el resultado por el mock
    req.flush(sancionats);
  });
});
