import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { LocalStorageService } from 'src/app/Shared/services/local-storage.service';
import { ArticleDTO } from '../../ordenances/model/article.dto';
import { ConcepteDTO } from '../model/concepte.dto';
import { TpActaDTO } from '../model/tpacta.dto';
import { SancioService } from './sancio.service';

@Injectable({
  providedIn: 'root',
})
export class SancioStorageService {
  constructor(
    private sancioService: SancioService,
    private localStorageService: LocalStorageService
  ) {}

  getTpActes(forceReload: boolean): Observable<TpActaDTO[]> {
    return new Observable((observer) => {
      let localobjs: TpActaDTO[] = this.localStorageService.get(
        'SancionsCalviaTipusActes'
      );
      if (forceReload || !localobjs) {
        this.sancioService.getTpActes().subscribe((objs: TpActaDTO[]) => {
          localobjs = objs;
          this.localStorageService.set('SancionsCalviaTipusActes', localobjs);
          observer.next(localobjs);
        }, (error)=>observer.error(error));
      } else {
        observer.next(localobjs);
      }
    });
  }

  getArticles(forceReload: boolean): Observable<ArticleDTO[]> {
    return new Observable((observer) => {
      let localobjs: ArticleDTO[] = this.localStorageService.get(
        'SancionsCalviaArticles'
      );
      if (forceReload || !localobjs) {
        this.sancioService.getArticles().subscribe((objs: ArticleDTO[]) => {
          localobjs = objs;
          this.localStorageService.set('SancionsCalviaArticles', localobjs);
          observer.next(localobjs);
        }, (error)=>observer.error(error));
      } else {
        observer.next(localobjs);
      }
    });
  }

  getConceptes(forceReload: boolean): Observable<ConcepteDTO[]> {
    return new Observable((observer) => {
      let localobjs: ConcepteDTO[] = this.localStorageService.get(
        'SancionsCalviaConceptes'
      );
      if (forceReload || !localobjs) {
        this.sancioService.getConceptes().subscribe((objs: ConcepteDTO[]) => {
          localobjs = objs;
          this.localStorageService.set('SancionsCalviaConceptes', localobjs);
          observer.next(localobjs);
        }, (error)=>observer.error(error));
      } else {
        observer.next(localobjs);
      }
    });
  }
}
