import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { apiconfig } from 'src/app/Shared/model/apitool';
import { ArticleDTO } from '../../ordenances/model/article.dto';
import { ActaDTO } from '../model/acta.dto';
import { ConcepteDTO } from '../model/concepte.dto';
import { FirmaActaDTO } from '../model/firma-acta.dto';
import { FotoActaDTO } from '../model/foto-acta.dto';
import { TpActaDTO } from '../model/tpacta.dto';

@Injectable({
  providedIn: 'root',
})
export class SancioService {
  private urlApi: string;

  constructor(private http: HttpClient) {
    this.urlApi = apiconfig.apiurl + 'acta';
  }

  getTpActes(): Observable<TpActaDTO[]> {
    return this.http.get<TpActaDTO[]>(this.urlApi + 'tipus');
  }

  getArticles(): Observable<ArticleDTO[]> {
    return this.http.get<ArticleDTO[]>(apiconfig.apiurl + 'article/articles');
  }

  getConceptes(): Observable<ConcepteDTO[]> {
    return this.http.get<ConcepteDTO[]>(this.urlApi + 'conceptes');
  }

  addActa(acta: ActaDTO): Observable<ActaDTO> {
    return this.http.post<ActaDTO>(this.urlApi, acta);
  }

  filterByDocument(document: string): Observable<ActaDTO[]> {
    return this.http.get<ActaDTO[]>(
      this.urlApi + '/filterByDocument?document=' + document
    );
  }

  filterByNumActa(year: string, numActa: string): Observable<ActaDTO[]> {
    return this.http.get<ActaDTO[]>(
      this.urlApi + '/filterByNumActa?year=' + year + '&numacta=' + numActa
    );
  }

  filterByKPersona(kpersona: number): Observable<ActaDTO[]> {
    return this.http.get<ActaDTO[]>(
      this.urlApi + '/filterByKPersona/' + kpersona
    );
  }

  getFirmes(kacta: string): Observable<FirmaActaDTO[]> {
    return this.http.get<FirmaActaDTO[]>(this.urlApi + '/firmes/' + kacta);
  }

  getFotos(kacta: string): Observable<FotoActaDTO[]> {
    return this.http.get<FotoActaDTO[]>(this.urlApi + '/fotos/' + kacta);
  }

  cobroEfectiu(acta: ActaDTO): Observable<ActaDTO> {
    return this.http.put<ActaDTO>(this.urlApi + '/cobroEfectiu', acta);
  }
}
