import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { AppState } from 'src/app/app.reducers';
import * as AuthAction from 'src/app/Modules/auth/actions';
import { checkAllowedOption } from '../../model/security-manager';

@Component({
  selector: 'app-headermenu',
  templateUrl: './headermenu.component.html',
  styleUrls: ['./headermenu.component.scss'],
})
export class HeadermenuComponent implements OnInit {
  showAuthSection: boolean;
  showNoAuthSection: boolean;
  showUsuariOption: boolean;
  showOrdenancesOption: boolean;
  showSancioOption: boolean;
  showInfoOption: boolean;
  username: string;

  constructor(private router: Router, private store: Store<AppState>) {
    this.showAuthSection = false;
    this.showUsuariOption = false;
    this.showOrdenancesOption = false;
    this.showSancioOption = false;
    this.showInfoOption = false;
    this.showNoAuthSection = true;
    this.username = '';
    this.store.select('auth').subscribe((auth) => {
      this.username = auth.authentication.username;
      this.showAuthSection = false;
      this.showNoAuthSection = true;
      if (auth.authentication.access_token) {
        this.showAuthSection = true;
        this.showNoAuthSection = false;
        this.showUsuariOption =
          !auth.authentication.canviPassword &&
          checkAllowedOption(auth.authentication, 'USUARI');
        this.showOrdenancesOption =
          !auth.authentication.canviPassword &&
          checkAllowedOption(auth.authentication, 'ORDENANCES');
        this.showSancioOption =
          !auth.authentication.canviPassword &&
          checkAllowedOption(auth.authentication, 'SANCIO');
        this.showInfoOption =
          !auth.authentication.canviPassword &&
          checkAllowedOption(auth.authentication, 'INFO');
      }
    });
  }

  ngOnInit(): void {}

  login(): void {
    this.router.navigateByUrl('auth/login');
  }

  infoSancions(): void {
    this.router.navigateByUrl('sancio/persones-list');
  }

  ordenances(): void {
    this.router.navigateByUrl('ordenances/ordenances-list');
  }

  sancions(): void {
    this.router.navigateByUrl('sancio/actes-list');
  }

  usuaris(): void {
    this.router.navigateByUrl('users/users-list');
  }

  changePassword(): void {
    this.router.navigateByUrl('auth/change-password');
  }

  logout(): void {
    this.store.dispatch(AuthAction.logout());
    this.router.navigateByUrl('auth/login');
  }

  help(): void {
    window.open('/SancionesCli/assets/doc/manual.pdf', '_blank');
  }
}
