import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
  UrlTree,
} from '@angular/router';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { AuthDTO } from 'src/app/Modules/auth/models/auth.dto';
import { AppState } from '../../app.reducers';
import { checkAllowedOption } from '../model/security-manager';

@Injectable({
  providedIn: 'root',
})
export class AuthGuard implements CanActivate {
  authentication?: AuthDTO;

  constructor(private router: Router, private store: Store<AppState>) {
    this.store.select('auth').subscribe((auth) => {
      if (auth.logged) {
        this.authentication = auth.authentication;
      }
    });
  }

  canActivate(
    route: ActivatedRouteSnapshot
  ):
    | Observable<boolean | UrlTree>
    | Promise<boolean | UrlTree>
    | boolean
    | UrlTree {
    if (this.authentication?.access_token) {
      if (route.data.option) {
        let isAllowed = checkAllowedOption(
          this.authentication,
          route.data.option
        );
        if (!isAllowed) {
          this.router.navigate(['home']);
          return false;
        }
      }
      return true;
    }

    this.router.navigate(['auth/login']);

    return false;
  }
}
