package controller.sancions;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.persistence.EntityManager;

import entity.reca.Rebut;
import entity.reca.Redsys;
import entity.sancions.Acta;
import entity.sancions.FirmaActa;
import entity.sancions.FotoActa;
import jaxb.redsys.Valinfo;
import utilities.JPAFactory;

public class ActaCtrl {

	public static Acta addActa(Acta acta) throws Exception {
		EntityManager em = null;
		try {
			acta.setData(new Date(System.currentTimeMillis()));
			em = JPAFactory.createEntityManager();
			em.getTransaction().begin();
			makeDenunciat(acta, em);
			makeDenunciant(acta, em);
			makeRebut(acta, em);

			if (acta.getDomacc() != null) {
				em.persist(acta.getDomacc());
			}

			acta.setNumActa(getNextNumActa(acta, em));
			em.persist(acta);
// Necesario si necesitamos recuperar kacta antes de commit			
			em.flush();
			makeFirmes(acta, em);
			makeFotos(acta, em);
			em.getTransaction().commit();
			return acta;
		} catch (Exception exc) {
			if (em != null && em.getTransaction().isActive())
				em.getTransaction().rollback();
			throw exc;
		} finally {
			if (em != null)
				em.close();
		}
	}

	public static Acta cobroEfectiu(Acta acta) throws Exception {
		EntityManager em = null;
		try {
			if (acta.getRebut() != null) {
				em = JPAFactory.createEntityManager();
				em.getTransaction().begin();
				Rebut reb = em.merge(acta.getRebut());
				reb.setEstat("C");
				reb.setTpcobro("E");
				reb.setDataCobro(new Date(System.currentTimeMillis()));
				em.getTransaction().commit();
				acta.setRebut(reb);
				return acta;
			} else
				throw new Exception("Acta sin recibo asociado");
		} 
		catch (Exception exc) {
			if (em!=null && em.getTransaction().isActive())
				em.getTransaction().rollback();
			throw exc;
		}
		finally {
			if (em != null)
				em.close();
		}
	}

	public static Acta prepareTpv(Acta acta, String orderid, Valinfo vinfo) throws Exception {
		EntityManager em = null;
		try {
			if (acta.getRebut() != null) {
				em = JPAFactory.createEntityManager();
				em.getTransaction().begin();
				Rebut reb = em.merge(acta.getRebut());
				reb.setMdredsys(vinfo.getMd());
				reb.setIdredsys(orderid);
				reb.setOperaciotpv(vinfo.getOrderinfo());
				Redsys redsys=new Redsys();
				redsys.setMdredsys(vinfo.getMd());
				redsys.setAcsurl(vinfo.getAcsurl());
				redsys.setPareq(vinfo.getPareq());
				em.merge(redsys);				
				em.getTransaction().commit();
				acta.setRebut(reb);
				return acta;
			} else
				throw new Exception("Acta sin recibo asociado");
		} 
		catch (Exception exc) {
			if (em!=null && em.getTransaction().isActive())
				em.getTransaction().rollback();
			throw exc;
		}
		finally {
			if (em != null)
				em.close();
		}
	}

	public static Acta cobroTpv(Acta acta, String operaciotpv) throws Exception {
		EntityManager em = null;
		try {
			if (acta.getRebut() != null) {
				em = JPAFactory.createEntityManager();
				em.getTransaction().begin();
				Rebut reb = em.merge(acta.getRebut());
				reb.setEstat("C");
				reb.setTpcobro("T");
				reb.setDataCobro(new Date(System.currentTimeMillis()));
				reb.setOperaciotpv(operaciotpv);
				em.getTransaction().commit();
				acta.setRebut(reb);
				return acta;
			} else
				throw new Exception("Acta sin recibo asociado");
		} 
		catch (Exception exc) {
			if (em!=null && em.getTransaction().isActive())
				em.getTransaction().rollback();
			throw exc;
		}
		finally {
			if (em != null)
				em.close();
		}
	}

	private static void makeDenunciat(Acta acta, EntityManager em) {
		if (acta.getDenunciat().getKpersona() != null) {
			em.merge(acta.getDenunciat());
		} else {
			em.persist(acta.getDenunciat());
		}
		if (acta.getDenunciat().getDomnotif().getKdomicili() != null) {
			em.merge(acta.getDenunciat().getDomnotif());
		} else {
			em.persist(acta.getDenunciat().getDomnotif());
		}

	}

	private static void makeDenunciant(Acta acta, EntityManager em) {
		if (acta.getDenunciant() != null) {
			acta.getDenunciant().setDomnotif(null);
			if (acta.getDenunciant().getKpersona() != null) {
				em.merge(acta.getDenunciant());
			} else
				em.persist(acta.getDenunciant());
		}
	}

	private static void makeRebut(Acta acta, EntityManager em) {
		if (acta.getArticle() != null && acta.getArticle().getImportSancio() != null) {
			Rebut rebut = new Rebut();
			rebut.setData(new Date(System.currentTimeMillis()));
			int year = getCurrentYear(rebut.getData());
			rebut.setAnio(year);
			rebut.setPadro("52");
			rebut.setOrdre("01");
			Integer maxnum = em.createNamedQuery("Rebut.maxnum", Integer.class).setParameter("year", year)
					.setParameter("padro", rebut.getPadro()).setParameter("ordre", rebut.getOrdre()).getSingleResult();
			rebut.setNumRebut(maxnum == null ? 1 : maxnum + 1);
			rebut.setEstat("P");
			rebut.setKpersona(acta.getDenunciat().getKpersona());
			rebut.setImportRebut(acta.getArticle().getImportSancio());
			em.persist(rebut);
			acta.setRebut(rebut);

		}
	}

	private static Integer getNextNumActa(Acta acta, EntityManager em) {
		Calendar inical = new GregorianCalendar();
		inical.setTime(acta.getData());
		inical.set(inical.get(java.util.Calendar.YEAR), 0, 1);
		Date dini = inical.getTime();

		Calendar fincal = new GregorianCalendar();
		fincal.setTime(acta.getData());
		fincal.set(fincal.get(java.util.Calendar.YEAR), 11, 31);
		Date dfi = fincal.getTime();

		Integer maxnum = em.createNamedQuery("Acta.maxnum", Integer.class).setParameter("inici", dini)
				.setParameter("fi", dfi).getSingleResult();
		return maxnum == null ? 1 : maxnum + 1;
	}

	private static void makeFirmes(Acta acta, EntityManager em) {
		if (acta.getFirmes() != null) {
			acta.getFirmes().forEach((firma) -> {
				firma.setKacta(acta.getKacta());
				em.persist(firma);
			});
		}

	}

	private static void makeFotos(Acta acta, EntityManager em) {
		if (acta.getFotos() != null) {
			acta.getFotos().forEach((foto) -> {
				foto.setKacta(acta.getKacta());
				em.persist(foto);
			});
		}

	}

	private static int getCurrentYear(Date data) {
		Calendar calendar = new GregorianCalendar();
		calendar.setTime(data);
		return calendar.get(Calendar.YEAR);
	}

	public static List<Acta> filterByDocument(String document) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			List<Acta> actes = em.createNamedQuery("Acta.findByDocument", Acta.class)
					.setParameter("document", document.toUpperCase()).getResultList();
			return actes;
		} finally {
			if (em != null)
				em.close();
		}
	}

	public static List<Acta> filterByKPersona(Integer kpersona) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			List<Acta> actes = em.createNamedQuery("Acta.findByKPersona", Acta.class).setParameter("kpersona", kpersona)
					.getResultList();
			return actes;
		} finally {
			if (em != null)
				em.close();
		}
	}

	public static List<FirmaActa> getFirmes(String kacta) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			List<FirmaActa> firmes = em.createNamedQuery("FirmaActa.firmes", FirmaActa.class)
					.setParameter("kacta", new Integer(kacta)).getResultList();
			return firmes;
		} finally {
			if (em != null)
				em.close();
		}
	}

	public static List<FotoActa> getFotos(String kacta) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			List<FotoActa> fotos = em.createNamedQuery("FotoActa.fotos", FotoActa.class)
					.setParameter("kacta", new Integer(kacta)).getResultList();
			return fotos;
		} finally {
			if (em != null)
				em.close();
		}
	}

	public static List<Acta> filterByNumActa(String year, String numActa) throws Exception {
		EntityManager em = null;
		try {
			Integer inumActa = Integer.valueOf(numActa);
			Integer iyear = Integer.valueOf(year);

			Calendar inical = new GregorianCalendar();
			inical.set(iyear, 0, 1);
			Date dini = inical.getTime();

			Calendar fincal = new GregorianCalendar();
			fincal.set(iyear, 11, 31);
			Date dfi = fincal.getTime();

			em = JPAFactory.createEntityManager();
			List<Acta> actes = em.createNamedQuery("Acta.findByNumActa", Acta.class).setParameter("numActa", inumActa)
					.setParameter("inici", dini).setParameter("fi", dfi).getResultList();
			return actes;
		} finally {
			if (em != null)
				em.close();
		}
	}

}
