package controller.security;

import java.util.Date;

import javax.crypto.SecretKey;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jws;
import io.jsonwebtoken.Jwts;
import utilities.KeyGenerator;

public class TokenCtrl {
	private String username;
	private String acces_token;
	private String refresh_token;
	
	
	public TokenCtrl(String username) {
		super();
		this.username = username;
	}


	public void generateTokens() throws Exception {
		issueToken();
		issueRefreshToken();
	}
	
	
	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getAcces_token() {
		return acces_token;
	}

	public void setAcces_token(String acces_token) {
		this.acces_token = acces_token;
	}

	public String getRefresh_token() {
		return refresh_token;
	}

	public void setRefresh_token(String refresh_token) {
		this.refresh_token = refresh_token;
	}

	private void issueToken() throws Exception {
		Date now=new Date();
		// 15 minutos de token.
		Date expTime=new Date(now.getTime()+900000);
		//10 segundos token para pruebas ->
		//Date expTime=new Date(now.getTime()+10000);


		SecretKey key = KeyGenerator.getTokenKey();

		acces_token = Jwts.builder().setIssuer("policiacalvia").setSubject(username.toUpperCase()).setExpiration(expTime).signWith(key).compact();
		// Issue a token (can be a random String persisted to a database or a JWT token)
		// The issued token must be associated to a user
		// Return the issued token
	}

	private void issueRefreshToken() throws Exception{
		Date now=new Date();
		//Una hora de refresh token
		Date expTime=new Date(now.getTime()+3600000);
		
		//Para pruebas 30 segundos
		//Date expTime=new Date(now.getTime()+30000);
		
		SecretKey key = KeyGenerator.getRefreshKey();

		refresh_token = Jwts.builder().setIssuer("policiacalvia").setSubject(username.toUpperCase()).setExpiration(expTime).signWith(key).compact();
		// Issue a token (can be a random String persisted to a database or a JWT token)
		// The issued token must be associated to a user
		// Return the issued token
	}
	
	private Jws<Claims> validateToken(String token, SecretKey key) throws Exception {
		// Check if the token was issued by the server and if it's not expired
		// Throw an Exception if the token is invalid
		Jws<Claims> jws = Jwts.parserBuilder()  // (1)
				.setSigningKey(key)         // (2)
				.build()                    // (3)
				.parseClaimsJws(token);
		return jws;	
	}
	
	
	public Jws<Claims> validateRefreshToken() throws Exception {
		// Check if the token was issued by the server and if it's not expired
		// Throw an Exception if the token is invalid
		SecretKey key = KeyGenerator.getRefreshKey();		
		return validateToken(refresh_token, key);		
	}
	
	public Jws<Claims> validateToken() throws Exception {
		// Check if the token was issued by the server and if it's not expired
		// Throw an Exception if the token is invalid		
		SecretKey key = KeyGenerator.getTokenKey();
		return validateToken(acces_token, key);		
	}

}
