package controller.security;

import java.util.List;

import javax.persistence.EntityManager;

import entity.security.MnOpcio;
import entity.security.OpcioRole;
import entity.security.Role;
import entity.security.Usuari;
import jaxb.security.CredentialsDTO;
import rest.security.Option;
import utilities.JPAFactory;

public class UsuariCtrl {

	public static Usuari checkPermissions(CredentialsDTO credentials) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			List<Usuari> users = em.createNamedQuery("Usuari.verifyUser", Usuari.class)
					.setParameter("entrada", credentials.getUsername().toUpperCase()).setParameter("password", credentials.getPassword()).getResultList();
			if (users == null || users.size() == 0)
				throw new Exception("Usuari no vàlid");
			else
				return users.get(0);
		} finally {
			if (em!=null) em.close();
		}
	}

	
	public static boolean isAuthorizedRequest(String entrada, Option option, String action) throws Exception {
		return isAuthorizedRequest(entrada, option.name(), action);
	}

	
	public static boolean isAuthorizedRequest(String entrada, String option_name, String action) throws Exception {
		boolean ok = false;
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			List<Usuari> users = em.createNamedQuery("Usuari.findByEntrada", Usuari.class)
					.setParameter("entrada", entrada.toUpperCase()).getResultList();
			if (users != null && users.size() > 0) {
				Usuari user = users.get(0);
				Role userrole = user.getRole();
				ok = checkRoleAllowed(userrole, option_name, action);
			}
		} finally {
			if (em!=null) em.close();
		}
		return ok;
	}	

	public static boolean userInRole(String entrada, String role_name) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			Usuari user = em.createNamedQuery("Usuari.userInRole", Usuari.class)
					.setParameter("entrada", entrada.toUpperCase())
					.setParameter("role",role_name).getSingleResult();
			return user!=null;
		} finally {
			if (em!=null) em.close();
		}
	}		
	
	private static boolean checkRoleAllowed(Role role, String optname, String action) {
		List<OpcioRole> user_options = role.getOpcioRoles();
		for (int cont = 0; user_options != null && cont < user_options.size(); cont++) {
			OpcioRole tmp_opcio = user_options.get(cont);
			MnOpcio mnopcio = tmp_opcio.getMnOpcio();
			if (mnopcio.getOpcio().equals(optname)) {
				if (action.equals("GET") && tmp_opcio.getSwselect())
					return true;
				if (action.equals("DELETE") && tmp_opcio.getSwdelete())
					return true;
				if (action.equals("POST") && tmp_opcio.getSwinsert())
					return true;
				if (action.equals("PUT") && tmp_opcio.getSwupdate())
					return true;
			}
		}
		return false;
	}

	
	public static Usuari addUsuari(Usuari usuari) throws Exception {
//		String cryptpass = DESCrypt.crypt(usuari.getPassword());
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			usuari.setEntrada(usuari.getEntrada().toUpperCase());		
			usuari.setCanviPassword(true);
//			usuari.setPassword(cryptpass);
			em.getTransaction().begin();
			em.persist(usuari);
			em.getTransaction().commit();
			return usuari;
		} finally {
			if (em!=null) em.close();
		}
	}
	
	public static Usuari getUsuari(Integer kusuari) throws Exception{
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			Usuari usuari=em.find(Usuari.class, kusuari);
			return usuari;
		} finally {
			if (em!=null) em.close();
		}
		
	}
	
	public static Usuari changePassword(CredentialsDTO credentials) throws Exception {
		Usuari usuari=checkPermissions(credentials);
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			em.getTransaction().begin();
			Usuari usuari_managed=em.merge(usuari);
			usuari_managed.setCanviPassword(false);
//			usuari_managed.setPassword(DESCrypt.crypt(credentials.getNewpassword()));
			usuari_managed.setPassword(credentials.getNewpassword());
			em.getTransaction().commit();
			return usuari;
		} finally {
			if (em!=null) em.close();
		}
	}
	
	public static Usuari saveUsuari(Usuari usuari) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			Usuari usuari_managed=getUsuari(usuari.getKusuari());
			em.getTransaction().begin();
			usuari_managed=em.merge(usuari_managed);
			usuari_managed.setNomcomplet(usuari.getNomcomplet());
			usuari_managed.setRole(usuari.getRole());
			if (!usuari.getPassword().equals("")) {
				usuari_managed.setCanviPassword(true);
//				usuari_managed.setPassword(DESCrypt.crypt(usuari.getPassword()));
				usuari_managed.setPassword(usuari.getPassword());
			}			
			em.getTransaction().commit();
			return usuari;			
		} finally {
			if (em!=null) em.close();
		}
	}
	
	public static List<Usuari> filterByName(String filter, boolean isSuperadmin) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			String namedQuery=isSuperadmin?"Usuari.findByNameFilterSuperadmin":"Usuari.findByNameFilter";
			List<Usuari> users = em.createNamedQuery(namedQuery, Usuari.class)
					.setParameter("filter", "%" + filter.toUpperCase() + "%").getResultList();
			return users;
		} finally {
			if (em!=null) em.close();
		}				
	}
	

	public static List<Role> listRoles(boolean isSuperAdmin) throws Exception {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			String namedQuery=isSuperAdmin?"Role.findAll":"Role.findAllNotSuperadmin";
			List<Role> roles = em.createNamedQuery(namedQuery, Role.class)
					.getResultList();
			return roles;
		} finally {
			if (em!=null) em.close();
		}		
		
	}
	
	
	/*
	 * public static boolean isAuthorizedRequest(String accio, Usuari user, String
	 * opcio) throws Exception { boolean isAuthorized=false; Role
	 * role=user.getRole(); List<MnOpcio> opcions=role.getMnOpcions();
	 * 
	 * 
	 * Mnopcusuari mnopc=getMnOpcUsuari(user, kFunc, kSubFunc, kProces); if
	 * (mnopc==null) { List<Usuari> parents=getUserGrups(user); for (int cont=0;
	 * cont<parents.size() && mnopc==null; cont++) { Usuari
	 * parent=parents.get(cont); return isAuthorizedRequest(accio, parent, kFunc,
	 * kSubFunc, kProces);
	 * 
	 * } } if (mnopc!=null) { if (accio.equals("INSERT") &&
	 * mnopc.getSwinsert().equals("S")) isAuthorized=true; if
	 * (accio.equals("UPDATE") && mnopc.getSwupdate().equals("S"))
	 * isAuthorized=true; if (accio.equals("DELETE") &&
	 * mnopc.getSwdelete().equals("S")) isAuthorized=true; if
	 * (accio.equals("SELECT") && mnopc.getSwselect().equals("S"))
	 * isAuthorized=true; } return isAuthorized; }
	 * 
	 * private static Mnopcusuari getMnOpcUsuari(Usuari user, String kFunc,String
	 * kSubFunc,String kProces) { EntityManager em=JPAFactory.createEntityManager();
	 * List<Mnopcusuari>
	 * opts=em.createNamedQuery("Mnopcusuari.findExact",Mnopcusuari.class).
	 * setParameter("kfunc", kFunc). setParameter("ksubfunc",kSubFunc).
	 * setParameter("kproces",kProces). setParameter("kusuari",user.getKusuari()).
	 * getResultList(); return opts==null || opts.size()==0?null:opts.get(0); }
	 * 
	 * private static List<Usuari> getUserGrups(Usuari user) { List<Usuari>
	 * parents=new ArrayList<Usuari>(); EntityManager
	 * em=JPAFactory.createEntityManager(); List<Grup>
	 * opts=em.createNamedQuery("Grup.findParents",Grup.class).
	 * setParameter("kusuari_membre", user.getKusuari()). getResultList(); if
	 * (opts!=null) { parents=opts.stream().map(grup->{ Usuari
	 * usr=em.find(Usuari.class, grup.getId().getKusuariGrup()); return usr;
	 * }).collect(Collectors.toList()); } return parents;
	 * 
	 */
}
