package rest.cadastre;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import entity.cadastre.Municipi;
import entity.cadastre.Pais;
import entity.cadastre.Provincia;
import entity.cadastre.TipCarrer;
import entity.cadastre.Via;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import rest.security.Option;
import rest.security.Secured;
import utilities.JPAFactory;

@Path("/Cadastre")
public class Cadastre {

	@GET
	@Secured({ Option.CADASTRE })
	@Path("/provincies")
	@Produces({ MediaType.APPLICATION_JSON })
	public Response getProvincies() {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			TypedQuery<Provincia> qprov = em.createNamedQuery("Provincia.findAll", Provincia.class);
			List<Provincia> lista = qprov.getResultList();
			return Response.ok(lista).build();
		} finally {
			em.close();
		}
	}

	@GET
	@Secured({ Option.CADASTRE })
	@Path("/paisos")
	@Produces({ MediaType.APPLICATION_JSON })
	public Response getPaisos() {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			TypedQuery<Pais> query = em.createNamedQuery("Pais.findAll", Pais.class);
			List<Pais> lista = query.getResultList();
			return Response.ok(lista).build();
		} finally {
			em.close();
		}
	}

	@GET
	@Secured({ Option.CADASTRE })
	@Path("/tipcarrers")
	@Produces({ MediaType.APPLICATION_JSON })
	public Response getTipCarrers() {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			TypedQuery<TipCarrer> query = em.createNamedQuery("TipCarrer.findAll", TipCarrer.class);
			List<TipCarrer> lista = query.getResultList();
			return Response.ok(lista).build();
		} finally {
			em.close();
		}
	}

	@GET
	@Secured({ Option.CADASTRE })
	@Path("/municipis")
	@Produces({ MediaType.APPLICATION_JSON })
	public Response getMunicipis(@QueryParam("kprovincia") Integer kprovincia) {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			TypedQuery<Municipi> query = em.createNamedQuery("Municipi.findByProvincia", Municipi.class).setParameter("kprovincia", kprovincia);
			List<Municipi> lista = query.getResultList();
			return Response.ok(lista).build();
		} finally {
			em.close();
		}
	}

	@GET
	@Secured({ Option.CADASTRE })
	@Path("/vies")
	@Produces({ MediaType.APPLICATION_JSON })
	public Response getVies(@QueryParam("kmunicipi") Integer kmunicipi) {
		EntityManager em = null;
		try {
			em = JPAFactory.createEntityManager();
			TypedQuery<Via> query = em.createNamedQuery("Via.findByMunicipi", Via.class).setParameter("kmunicipi", kmunicipi);
			List<Via> lista = query.getResultList();
			return Response.ok(lista).build();
		} finally {
			em.close();
		}
	}
	
	
	
}
