package rest.comu;

import java.util.List;

import controller.comu.PersonaCtrl;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import rest.security.Option;
import rest.security.Secured;

@Path("/persona")
public class Persona {

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findById/{kpersona}")
	@Secured({ Option.INFO })
	public Response findById(@PathParam("kpersona") Integer kpersona) {
		try {
			entity.comu.Persona pers = PersonaCtrl.findById(kpersona);
			return (pers==null?Response.ok():Response.ok(pers)).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}	
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findByNif/{document}")
	@Secured({ Option.SANCIO })
	public Response findByNif(@PathParam("document") String document) {
		try {
			entity.comu.Persona pers = PersonaCtrl.findByNif(document);
			return (pers==null?Response.ok():Response.ok(pers)).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findByPassaport/{document}")
	@Secured({ Option.SANCIO })
	public Response findByPassaport(@PathParam("document") String document) {
		try {
			entity.comu.Persona pers = PersonaCtrl.findByPassaport(document);
			return (pers==null?Response.ok():Response.ok(pers)).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findByDocument/{document}")
	@Secured({ Option.SANCIO })
	public Response findByDocument(@PathParam("document") String document) {
		try {
			entity.comu.Persona pers = PersonaCtrl.findByDocument(document);
			return (pers==null?Response.ok():Response.ok(pers)).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/filterByName")
	@Secured({ Option.INFO })
	public Response getPersonesByName(@QueryParam("nom") String filter) {
		try {
			if (filter.trim().length()<3) {
				Response.status(Response.Status.BAD_REQUEST).entity("El nombre a buscar debe tener al menos 3 caracteres").build();
			}
			List<entity.comu.Sancionat> persones = PersonaCtrl.findByNom(filter);
			return Response.ok(persones).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}	


	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findSancionatByDocument/{document}")
	@Secured({ Option.INFO })
	public Response getSancionatByDocument(@PathParam("document") String document) {
		try {
			entity.comu.Sancionat persona = PersonaCtrl.findSancionatByDocument(document);
			return Response.ok(persona).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}	
	
	
}
