package rest.sancions;

import java.util.List;

import controller.sancions.ActaCtrl;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import rest.security.Option;
import rest.security.Secured;

@Path("/acta")
public class Acta {
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Secured({ Option.SANCIO })
	public Response addActa(entity.sancions.Acta acta) {
		try {
			acta = ActaCtrl.addActa(acta);
			return Response.ok(acta).build();
		} catch (Exception exc) {
			exc.printStackTrace();
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Secured({ Option.SANCIO })
	@Path("/cobroEfectiu")
	public Response cobroEfectiu(entity.sancions.Acta acta) {
		try {
			acta = ActaCtrl.cobroEfectiu(acta);
			return Response.ok(acta).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}	
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/filterByDocument")
	@Secured({ Option.SANCIO })
	public Response getSancionsByDocument(@QueryParam("document") String document) {
		try {
			List<entity.sancions.Acta> actes = ActaCtrl.filterByDocument(document);
			return Response.ok(actes).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/filterByNumActa")
	@Secured({ Option.SANCIO })
	public Response getSancionsByNumActa(@QueryParam("year") String year, @QueryParam("numacta") String numActa) {
		try {
			List<entity.sancions.Acta> actes = ActaCtrl.filterByNumActa(year, numActa);
			return Response.ok(actes).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/filterByKPersona/{kpersona}")
	@Secured({ Option.INFO })
	public Response getSancionsByKPersona(@PathParam("kpersona") Integer kpersona) {
		try {
			List<entity.sancions.Acta> actes = ActaCtrl.filterByKPersona(kpersona);
			return Response.ok(actes).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}	
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/firmes/{kacta}")
	@Secured({ Option.SANCIO })
	public Response getFirmes(@PathParam("kacta") String kacta) {
		try {
			List<entity.sancions.FirmaActa> firmes = ActaCtrl.getFirmes(kacta);
			return Response.ok(firmes).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/fotos/{kacta}")
	@Secured({ Option.SANCIO })
	public Response getFotos(@PathParam("kacta") String kacta) {
		try {
			List<entity.sancions.FotoActa> firmes = ActaCtrl.getFotos(kacta);
			return Response.ok(firmes).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}	
	
}
