package rest.sancions;

import java.util.List;


import controller.sancions.ArticleCtrl;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import rest.security.Option;
import rest.security.Secured;

@Path("/article")
public class Article {
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Secured({ Option.ORDENANCES })
	public Response addArticle(entity.sancions.Article article) {
		try {
			article = ArticleCtrl.addArticle(article);
			return Response.ok(article).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Secured({ Option.ORDENANCES })
	public Response saveArticle(entity.sancions.Article article) {
		try {
			article = ArticleCtrl.saveArticle(article);
			return Response.ok(article).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("{karticle}")
	@Secured({ Option.ORDENANCES })
	public Response getArticle(@PathParam("karticle") Integer karticle) {
		try {
			entity.sancions.Article article = ArticleCtrl.getArticle(karticle);
			return Response.ok(article).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@DELETE
	@Path("{karticle}")
	@Secured({ Option.ORDENANCES })
	public Response deleteArticle(@PathParam("karticle") Integer karticle) {
		try {
			ArticleCtrl.deleteArticle(karticle);
			return Response.ok().build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/filterByName")
	@Secured({ Option.ORDENANCES })
	public Response getArticlesByName(@QueryParam("filter") String filter) {
		try {
			List<entity.sancions.Article> articles = ArticleCtrl.filterByName(filter);
			return Response.ok(articles).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/articles")
	@Secured({ Option.ORDENANCES })
	public Response getArticles() {
		try {
			List<entity.sancions.Article> articles = ArticleCtrl.getArticles();
			return Response.ok(articles).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}
	
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/filterByCode")
	@Secured({ Option.ORDENANCES })
	public Response getArticlesByCode(@QueryParam("filter") String filter) {
		try {
			List<entity.sancions.Article> articles = ArticleCtrl.filterByCode(filter);
			return Response.ok(articles).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}
	
	
}
