package rest.security;


import java.util.List;

import controller.security.TokenCtrl;
import controller.security.UsuariCtrl;
import entity.security.OpcioRole;
import entity.security.Usuari;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import jaxb.security.AuthDTO;
import jaxb.security.CredentialsDTO;

@Path("/authentication")
public class Authentication {

	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response authenticateUser(CredentialsDTO credentials) {


		try {

			// Authenticate the user using the credentials provided
			Usuari usuari=UsuariCtrl.checkPermissions(credentials);

			TokenCtrl tokenCtrl=new TokenCtrl(credentials.getUsername());
			tokenCtrl.generateTokens();
			// Issue a token for the user
			
			List<OpcioRole> opcioRole=usuari.getRole().getOpcioRoles();

			AuthDTO auth=new AuthDTO(credentials.getUsername(), tokenCtrl.getAcces_token(), tokenCtrl.getRefresh_token(), usuari.isCanviPassword(), opcioRole);

			// Return the token on the response
			return Response.ok(auth).header("Access-Control-Allow-Origin", "*").
					header("Access-Control-Allow-Credentials", "true").
					header("Access-Control-Allow-Headers","origin, content-type, accept, authorization").
					header("Access-Control-Allow-Methods","GET, POST, PUT, DELETE, OPTIONS, HEAD").build();

		} catch (Exception e) {
			return Response.status(Response.Status.FORBIDDEN).entity("Usuari / Password incorrecte").build();
		}      
	}

	@POST
	@Path("/refreshToken")
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response refreshToken(AuthDTO auth) {
		try {

			// Authenticate the user using the credentials provided
			TokenCtrl tokenCtrl=new TokenCtrl(auth.getUsername());
			tokenCtrl.setRefresh_token(auth.getRefresh_token());
			tokenCtrl.validateRefreshToken();
			tokenCtrl.generateTokens();

			AuthDTO newauth=new AuthDTO(auth.getUsername(), tokenCtrl.getAcces_token(), tokenCtrl.getRefresh_token());

			// Return the token on the response
			return Response.ok(newauth).build();

		} catch (Exception e) {
			return Response.status(Response.Status.FORBIDDEN).entity("Temps de connexió caducat").build();
		}      
	}	
	
	private void authenticate_by_api(String username, String password) throws Exception {
/*
		Client client=ClientBuilder.newClient();

		LoginDTO loginreq=new LoginDTO(username, password, 0);

		Response response=client.
				target("https://extranet2.calvia.com/VinfoPolWebApi/api/Login").request(MediaType.TEXT_PLAIN_TYPE).post(Entity.entity(loginreq, MediaType.APPLICATION_JSON));
		
		String loginresp=response.readEntity(String.class);
		
		System.out.println(loginresp);
		
		if (response.getStatus()==200 && loginresp.indexOf("{\"Exito\":true")>=0)  return;

		throw new Exception("Forbidden");
		// Authenticate against a database, LDAP, file or whatever
		// Throw an Exception if the credentials are invalid
		 * 
		 */
		
	}




}