package rest.security;

import java.util.List;

import javax.persistence.EntityManager;

import controller.security.UsuariCtrl;
import entity.security.Role;
import entity.security.Usuari;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.SecurityContext;
import jaxb.security.CredentialsDTO;

@Path("/usuari")
public class User {
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Secured({ Option.USUARI })
	public Response addUser(Usuari usuari) {
		try {
			usuari = UsuariCtrl.addUsuari(usuari);
			usuari.setPassword("");
			return Response.ok(usuari).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@POST
	@Path("/changePassword")
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Secured({ Option.PASSWORD })
	public Response changePassword(CredentialsDTO credentials) {
		try {
			Usuari usuari = UsuariCtrl.changePassword(credentials);
			usuari.setPassword("");
			return Response.ok(usuari).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@PUT
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Secured({ Option.USUARI })
	public Response saveUsuari(Usuari usuari) {
		try {
			usuari = UsuariCtrl.saveUsuari(usuari);
			usuari.setPassword("");
			return Response.ok(usuari).build();
		} catch (Exception exc) {
			exc.printStackTrace();
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("{kusuari}")
	@Secured({ Option.USUARI })
	public Response getUsuari(@PathParam("kusuari") Integer kusuari) {
		try {
			Usuari usuari = UsuariCtrl.getUsuari(kusuari);
			return Response.ok(usuari).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/list")
	@Secured({ Option.USUARI })
	public Response getUsuaris(@Context SecurityContext sc, @QueryParam("filter") String filter) {
		try {
			boolean isSuperadmin=sc.isUserInRole("SUPERADMIN");
			List<Usuari> users = UsuariCtrl.filterByName(filter, isSuperadmin);
			return Response.ok(users).build();
		} catch (Exception exc) {
			exc.printStackTrace();
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/listroles")
	@Secured({ Option.USUARI })
	public Response getRoles(@Context SecurityContext sc) {
		try {
			boolean isSuperadmin=sc.isUserInRole("SUPERADMIN");
			List<Role> roles = UsuariCtrl.listRoles(isSuperadmin);
			return Response.ok(roles).build();
		} catch (Exception exc) {
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(exc.getMessage()).build();
		}
	}

}
