package utilities;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import entity.reca.Rebut;

public class BarCodeEABQ38Format521 {

	String iCode = null;
	String iApp	 = "90";
	String iTpFormat = "521";
	String iEmisora = null;
	String iModalitat = "2";
	String iReferencia = null;
	String iIdentificacio = null;
	String iImport = null;
	String iParitat = "0";

	public BarCodeEABQ38Format521(int padro,int any,int ordre,int numRebut,String idTrib,java.util.Date dataLimit,java.math.BigDecimal imp) throws Exception {
		this("070117",padro,any,ordre,numRebut,idTrib,dataLimit,imp);
	}

	public BarCodeEABQ38Format521(int padro,int any,int ordre,int numRebut,String idTrib,java.util.Date dataLimit,java.math.BigDecimal imp,String prefixIdentificacio) throws Exception {
		this("070117",padro,any,ordre,numRebut,idTrib,dataLimit,imp,prefixIdentificacio);
	}

	public BarCodeEABQ38Format521(String emissora,int padro,int any,int ordre,int numRebut,String idTrib,java.util.Date dataLimit,java.math.BigDecimal imp) throws Exception {
		this(emissora,padro,any,ordre,numRebut,idTrib,dataLimit,imp,"1");  // Prefix "1": sense recàrrec
	}
	
	public BarCodeEABQ38Format521(String emissora,int padro,int any,int ordre,int numRebut,String idTrib,java.util.Date dataLimit,java.math.BigDecimal imp,String prefixIdentificacio) throws Exception {

		iEmisora = emissora; // "070117";

		String padro_2d		= formatNum(new Integer(padro).toString(), 2, 0);	// dos digits padró 
		String any_2d   	= formatNum(new Integer(any % 100).toString(),2,0);
		String ordre_1d 	= formatNum(new Integer(ordre % 10).toString(),1,0);
		String numrebut_5d	= formatNum(new Integer(numRebut).toString(),5,0);
		
		String referenciap 	= 	padro_2d +
								any_2d +
								ordre_1d +
								numrebut_5d; 

		boolean swDosDigitsOrdre = isPadro2DigitsOrdre(emissora,padro_2d);
			
		if (swDosDigitsOrdre) {	
			String ordre_2d 	= formatNum(new Integer(ordre % 100).toString(),2,0);
			String numrebut_4d	= formatNum(new Integer(numRebut).toString(),4,0);
			referenciap 	= 	padro_2d +
								any_2d +
								ordre_2d +
								numrebut_4d; 
		}

		Calendar calendar = new GregorianCalendar();
		calendar.setTime(dataLimit);
		int year=calendar.get(Calendar.YEAR);	
		
		String any_data_limit_1d = "" + (year % 10);
		
		String dies_data_limit_juliana = formatNum(""+calcDiesJuliana(dataLimit),3,0);
		
		
		iIdentificacio = 	prefixIdentificacio +
							formatNum(idTrib, 3, 0) + // codi tribut
							any_2d + // exercici devengo en dos digits
							any_data_limit_1d + // darrer digit any data limit
							dies_data_limit_juliana;	// 3 digits
		
		String referencia_dgc = calc_refDGC(iEmisora,referenciap,iIdentificacio,imp);

		iReferencia    = referenciap + referencia_dgc;
		
		iImport  = formatNum(imp.toString(), 8, 2);
		iCode    = iApp+iTpFormat+iEmisora+iReferencia+iIdentificacio+iImport+iParitat;
	}
	
	public BarCodeEABQ38Format521(String emisora,String referencia,String identificacio,String imp) throws Exception {
		iEmisora = emisora;
		iReferencia = referencia;
		iIdentificacio = identificacio;
		iImport = imp;
		iCode = iApp + iTpFormat + iEmisora + iReferencia + iIdentificacio + iImport + iParitat;
		if (iCode.length()!=42) throw new Exception("llargaria de dades incorrecte");
	}
	
	public String getTpFormat() {
		return iTpFormat;
	}
	
	private void decode() {
		iApp 		= iCode.substring(0,2);
		iTpFormat 	= iCode.substring(2,5);
		iEmisora  	= iCode.substring(5,11);
		iReferencia = iCode.substring(11,23);
		iIdentificacio = iCode.substring(23,33);
		iImport		= iCode.substring(33,41);
		iParitat	= iCode.substring(41);
	}
	
	public BarCodeEABQ38Format521(String code) throws Exception {
		iCode = code;
		if (iCode.length()!=42) throw new Exception("llargaria de dades incorrecte");
		decode();
		if (!iTpFormat.equals("521")) throw new Exception("Format del codi de barres incorrecte");
	}
		
	public BarCodeEABQ38Format521(String emissora,String referenciacomp,String identificaciocomp,java.math.BigDecimal imp) throws Exception {
		
		String imps	= 	formatNum(imp.toString(), 8, 2);
		
		iCode 		= 	iApp +
						iTpFormat +
						emissora +
						referenciacomp +
						identificaciocomp +
						imps +
						iParitat;

		if (iCode.length()!=42) throw new Exception("llargaria de dades incorrecte");
		
		decode();
	}
	
	public String getEmisora() {
		return iEmisora;
	}
	
	public String getModalitat() {
		return iModalitat;
	}
	
	public String getReferencia() {
		return iReferencia;
	}
	
	public String getIdentificacio() {
		return iIdentificacio;
	}
	
	public String getImport() {
		return iImport;
	}
	
	public String toString() {
		return iCode;
	}
	
	
	private static int calcDiesJuliana(java.util.Date data) {
	
		java.util.Calendar cal = java.util.Calendar.getInstance();
		cal.setTime(data);
		int jul = cal.get(cal.DAY_OF_YEAR);
		
	    return jul;
	}

	private static String calc_refDGC(String emisora,String referenciap,String identificacio,java.math.BigDecimal imp) {
	    long emis 	= Long.parseLong(emisora);
	    long refer 	= Long.parseLong(referenciap);
	    long ident 	= Long.parseLong(identificacio) + imp.multiply(new java.math.BigDecimal("100")).longValue();

	    long aux = (emis*76l+refer*9l+(ident-1l)*55l);// /97;
	    java.math.BigDecimal auxb = new java.math.BigDecimal(String.valueOf(aux));
	    auxb = auxb.divide(new java.math.BigDecimal("97"), 4, java.math.BigDecimal.ROUND_HALF_UP);
	    long aux2 = auxb.multiply(new java.math.BigDecimal("100")).longValue();
	    aux2 %= 100;
	    int ndd = 99 - (int)aux2; // 99 - ((truncate(aux,2)-truncate(aux))*100)

	    java.text.NumberFormat formatter = new java.text.DecimalFormat("00");
	    String dd = formatter.format(ndd);
	    return dd;
	}
	
	
	

	
	public Rebut getIdRebutFromCodeBar() throws Exception {
		Rebut ir = new Rebut();
		
		ir.setPadro(getReferencia().substring(0,2));
		ir.setAnio(2000 + Integer.parseInt(getReferencia().substring(2,4)));
		
		boolean isOrdre2Digits = isPadro2DigitsOrdre(this.getEmisora(),ir.getPadro());

		if (isOrdre2Digits) { // Padrons Liq/AutoLiq ...
			ir.setOrdre(getReferencia().substring(4,6));
			ir.setNumRebut(Integer.parseInt(getReferencia().substring(6,10)));
		} else {
			ir.setOrdre("0"+getReferencia().substring(4,5));
			ir.setNumRebut(Integer.parseInt(getReferencia().substring(5,10)));
		}
		
		return ir;
	}

	private static List idTributsNoGeneratAjuntament = Arrays.asList(new String[]{"000" });
	
	public String getIdTribut() {
		String identificacio = getIdentificacio();
		return identificacio.substring(1,4);
	}	
	
	public boolean isCodeBarGeneratPerAjuntament() {
		String kTribut = this.getIdTribut();
		return !idTributsNoGeneratAjuntament.contains(kTribut);
	}
	
	public java.util.Date getMaxDataFromCodeBar() throws Exception {
		
		String identificacio = getIdentificacio();
		
		String strAnyDevengo = identificacio.substring(4,6); 
		String strAnyMaxData = identificacio.substring(6,7);
		String strMaxDataJuliana = identificacio.substring(7); 

		if (strMaxDataJuliana.length()!=3) throw new Exception("Error en el format de la identificació. Data Juliana no quadra.");
		
		int any2dDevengo = Integer.valueOf(strAnyDevengo);
		int any1dMaxData = Integer.valueOf(strAnyMaxData);
		int maxDataJuliana = Integer.valueOf(strMaxDataJuliana);
		
		int any = 2000 + any2dDevengo; // Quan arribi el 2100 no sé qui hi serà de noltros ...
		if (this.isCodeBarGeneratPerAjuntament()) {
			Rebut	idRebut = getIdRebutFromCodeBar();			
			any = idRebut.getAnio().intValue();	// Any del rebut = any devengo
			if ((any%100)!=any2dDevengo) throw new Exception("Error en el contingut de la identificació. Any no quadra.");
		}
		
		if (maxDataJuliana>366 || maxDataJuliana<1) throw new Exception("Error en el contingut de la identificació. Data Juliana incorrecte.");

		int r = any1dMaxData - (any%10);
		if (r<0) r = 10 + r;
		int anyMaxData = any + r;
		
		java.util.Calendar cMaxData = java.util.Calendar.getInstance(new Locale("ES"));
		cMaxData.clear();
		cMaxData.set(cMaxData.YEAR, anyMaxData);
		cMaxData.set(cMaxData.DAY_OF_YEAR, maxDataJuliana);

		java.util.Date maxData = cMaxData.getTime();

		return maxData;
	}
		
	// Per l'emissora municipal, només aquests padrons tenen 2 digits d'ordre:
	static java.util.Vector<String> padro2Digits = new java.util.Vector<String>(
													Arrays.asList(new String[]{
																"70",
																"23",
																"54"})
													);  

	static public boolean isPadro2DigitsOrdre(String padro) {
		return isPadro2DigitsOrdre("070117",padro);
	}	
	
	static public boolean isPadro2DigitsOrdre(String emissora,String padro) {
		boolean is1digitOnly = (emissora.equals("070117") && !padro2Digits.contains(padro)) || 
								emissora.equals("072515");  // Calvià2000: Un dígit d'ordre
								// (emissora.equals("074913")); -> IMEB: dos dígits  
		return !is1digitOnly;
	}
	
	public boolean areDigitsValid() { // Comprova que els dígits siguin correctes
		return areDigitsValid(null);
	}
	
	public boolean areDigitsValid(String []adgc) { // Comprova que els dígits siguin correctes
		String ref = this.getReferencia();
		String referenciap = ref.substring(0,ref.length()-2);
		String referencia_dgc =  ref.substring(ref.length()-2);
		String dgc = calc_refDGC(this.getEmisora(),referenciap,this.getIdentificacio(),new java.math.BigDecimal(this.getImport()).divide(new java.math.BigDecimal("100")));
		if (adgc!=null && adgc.length>=2) {
			adgc[0] = referencia_dgc; 
			adgc[1] = dgc;
		}
		return referencia_dgc.equals(dgc);
	}
	
	
	public static void main(String args[]) throws Exception {
		BarCodeEABQ38Format521 b = null;
		String resp[] = new String[2];
		
		b =
			new BarCodeEABQ38Format521("070117", "701702110146", "1070177171", new java.math.BigDecimal("96.98"));
		b.areDigitsValid(resp);

		b =
			new BarCodeEABQ38Format521("070117", "011712146686", "5001177228", new java.math.BigDecimal("1507.04"));

		b.areDigitsValid(resp);

		
		b =
			new BarCodeEABQ38Format521("070117", "141710071624", "5014177228", new java.math.BigDecimal("606.53"));
		b.areDigitsValid(resp);


		b =
			new BarCodeEABQ38Format521("905210701171417100716245014177228000606530");
		b.areDigitsValid(resp);
	
		b =
			new BarCodeEABQ38Format521("905220701171417100716245014177228000606530");
		b.areDigitsValid(resp);
	}

	private String formatNum(String nro, int len, int ndec) {
		if (nro==null) nro="";
		String	str = repStr('0',len), psenc="", pdec="";
		int		ppunt;

		ppunt = nro.indexOf('.');
		if (ppunt==-1) {
	                // Arreglo 21/8/2002: no es calculava bé la llargària: len-nro.length no servia si hi havia decimals
	                // -> no s'havia tengut en compte que el nro no vengués amb decimals però tengués formateig amb decimals
			psenc = repStr('0',len-ndec-nro.length()) + nro;
			pdec = repStr('0',ndec);
		} else {
			psenc = nro.substring(0,ppunt);
			psenc = repStr('0',(len - ndec) - psenc.length()) + psenc;
			if (ndec>0) {
				pdec = nro.substring(ppunt+1,nro.length());
				pdec = pdec + repStr('0',ndec - pdec.length());
			}
		}
		return 	psenc + pdec;
	}
	
	private String repStr(char c,int len ) {
		StringBuffer s = new StringBuffer(len);
		for (int i=0;i<len;i++)
			s.append(c);
		return s.toString();
	}	
	

	
	
}
