package test.endpoints;

import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.test.JerseyTest;
import org.glassfish.jersey.test.TestProperties;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import jakarta.ws.rs.client.Entity;
import jakarta.ws.rs.core.Application;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import jaxb.security.AuthDTO;
import jaxb.security.CredentialsDTO;
import rest.security.Authentication;
import utilities.Config;

import static org.junit.Assert.*;

public class AuthenticationTest  {
	private static JerseyTest jerseyTest;
		

    @BeforeClass
    public static void beforeClass() throws Exception {
    	Config.persistentUnitName="TestSancionesApi";
    	initJerseyTest(); 
        jerseyTest.setUp();
    }	
    
    @AfterClass
    public static void afterClass() throws Exception {
    	jerseyTest.tearDown();
    }
	
	
    private static void initJerseyTest() {
        jerseyTest = new JerseyTest() {
        	@Override
            protected Application configure() {
                // Find first available port.
                enable(TestProperties.LOG_TRAFFIC);
                enable(TestProperties.DUMP_ENTITY); 
                return new ResourceConfig(Authentication.class);
            }
        };
      }    
    
	@Test
	public void testCredencialesCorrectas( ) {
		CredentialsDTO credentials=new CredentialsDTO("admin","Admin2022");
		Entity<CredentialsDTO> credentialsEntity = Entity.entity(credentials, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("authentication").request(MediaType.APPLICATION_JSON_TYPE)
		        .post(credentialsEntity);
		assertEquals(200, response.getStatus());
	}

	@Test
	public void testCredencialesErroneas( ) {
		CredentialsDTO credentials=new CredentialsDTO("admin","admin");
		Entity<CredentialsDTO> credentialsEntity = Entity.entity(credentials, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("authentication").request(MediaType.APPLICATION_JSON_TYPE)
		        .post(credentialsEntity);
		assertEquals(403, response.getStatus());
	}

	@Test
	public void testRefreshTokenCorrecto( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Entity<AuthDTO> authEntity = Entity.entity(auth, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("authentication/refreshToken").request(MediaType.APPLICATION_JSON_TYPE)
		        .post(authEntity);
		assertEquals(200, response.getStatus());
	}
	
	



}
