package test.endpoints;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertTrue;

import java.util.List;

import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.test.JerseyTest;
import org.glassfish.jersey.test.TestProperties;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import entity.sancions.Article;
import entity.security.Role;
import entity.security.Usuari;
import jakarta.ws.rs.client.Entity;
import jakarta.ws.rs.core.Application;
import jakarta.ws.rs.core.GenericType;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import jaxb.security.AuthDTO;
import jaxb.security.CredentialsDTO;
import rest.security.Authentication;
import rest.security.User;
import utilities.Config;

public class OrdenancesTest {
	private static JerseyTest jerseyTest;
	

    @BeforeClass
    public static void beforeClass() throws Exception {
    	Config.persistentUnitName="TestSancionesApi";
    	initJerseyTest(); 
        jerseyTest.setUp();
    }	
	
    @AfterClass
    public static void afterClass() throws Exception {
    	jerseyTest.tearDown();
    }
	    
    private static void initJerseyTest() {
        jerseyTest = new JerseyTest() {
        	@Override
            protected Application configure() {
                // Find first available port.
                enable(TestProperties.LOG_TRAFFIC);
                enable(TestProperties.DUMP_ENTITY); 
                return new ResourceConfig(Authentication.class, rest.sancions.Article.class);
            }
        };
      }    
    
	@Test
	public void testAnadirArticulo( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Article article=new Article();
		article.setCodi("40");
		article.setDescripcio("ARTICULO 1");
		Entity<Article> articleEntity = Entity.entity(article, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("article").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .post(articleEntity);
		Article newArticle=response.readEntity(Article.class);
		assertEquals(200, response.getStatus());
		assertTrue(newArticle.getKarticle()!=null);
	}
	

	@Test
	public void testModificarArticulo( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Response response = jerseyTest.target("article/1").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();
		Article article=response.readEntity(Article.class);
		article.setDescripcio("CAMBIA DESCRIPCION");
		Entity<Article> articleEntity = Entity.entity(article, MediaType.APPLICATION_JSON_TYPE);
		Response updateResponse = jerseyTest.target("article").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .put(articleEntity);
		Article updatedArticle=updateResponse.readEntity(Article.class);
		assertEquals(200, updateResponse.getStatus());
		assertEquals(article.getKarticle(), updatedArticle.getKarticle());
		assertEquals(updatedArticle.getDescripcio(),"CAMBIA DESCRIPCION" );
	}

	
	@Test
	public void testFiltrarCodigo( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Response response = jerseyTest.target("article/filterByCode").queryParam("filter", "12").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();
		List<Article> articles=response.readEntity(new GenericType<List<Article>>() {
        });
		
		assertEquals(200, response.getStatus());
		assertEquals(true, articles.size()>0);
	}
	

	@Test
	public void testFiltrarNombre( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Response response = jerseyTest.target("article/filterByName").queryParam("filter", "DROGAS").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();
		List<Article> articles=response.readEntity(new GenericType<List<Article>>() {
        });
		
		assertEquals(200, response.getStatus());
		assertEquals(true, articles.size()>0);
	}
	
	
	
}
