package test.endpoints;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertTrue;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.test.JerseyTest;
import org.glassfish.jersey.test.TestProperties;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import entity.comu.Persona;
import entity.sancions.Acta;
import entity.sancions.Article;
import entity.security.Role;
import entity.security.Usuari;
import jakarta.ws.rs.client.Entity;
import jakarta.ws.rs.core.Application;
import jakarta.ws.rs.core.GenericType;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import jaxb.security.AuthDTO;
import jaxb.security.CredentialsDTO;
import rest.security.Authentication;
import rest.security.User;
import utilities.Config;

public class SancionsTest {
	private static JerseyTest jerseyTest;
	private static Persona pers;
	private static Article art;
	
	

    @BeforeClass
    public static void beforeClass() throws Exception {
    	Config.persistentUnitName="TestSancionesApi";
    	initJerseyTest(); 
        jerseyTest.setUp();
        initData();
    }
    @AfterClass
    public static void afterClass() throws Exception {
    	jerseyTest.tearDown();
    }
	
    private static void initJerseyTest() {
        jerseyTest = new JerseyTest() {
        	@Override
            protected Application configure() {
                // Find first available port.
                enable(TestProperties.LOG_TRAFFIC);
                enable(TestProperties.DUMP_ENTITY); 
                return new ResourceConfig(Authentication.class, rest.sancions.Article.class, rest.sancions.Acta.class, rest.comu.Persona.class);
            }
        };
      }    
    
	@Test
	public void testAnadirActa( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Acta acta=new Acta();
		acta.setArticle(art);
		acta.setDenunciat(pers);
		acta.setData(new Date(System.currentTimeMillis()));
		acta.setDescripcio("TEST AÑADIR ACTA");
		acta.setSitukvia(3012);
		acta.setSituampliacio("");
		acta.setSitubloc("");
		acta.setSituescala("");
		acta.setSituktipcarrer("CL");
		acta.setSitulletra("");
		acta.setSitunomvia("CA NA CUCO");
		acta.setSitunro(100);
		acta.setSituplanta("");
		acta.setSituporta("");
		acta.setSituportal("");
		acta.setSituampliacio("");
		Entity<Acta> actaEntity = Entity.entity(acta, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("acta").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .post(actaEntity);
		Acta newActa=response.readEntity(Acta.class);
		assertEquals(200, response.getStatus());
		assertTrue(newActa.getKacta()!=null);
		assertTrue(newActa.getRebut()!=null);
	}
	
	@Test
	public void testCobrarActa( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Calendar calendar = new GregorianCalendar();
		calendar.setTimeInMillis(System.currentTimeMillis());
		Integer year=calendar.get(Calendar.YEAR);		
		Response response_acta = jerseyTest.target("acta/filterByNumActa").queryParam("year", year.toString()).queryParam("numacta", "1").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();
		List<Acta> actes=response_acta.readEntity(new GenericType<List<Acta>>() {});		
		Acta acta=actes.get(0);
		Entity<Acta> actaEntity = Entity.entity(acta, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("acta/cobroEfectiu").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .put(actaEntity);
		Acta newActa=response.readEntity(Acta.class);
		assertTrue(newActa.getRebut()!=null);
		assertTrue(newActa.getRebut().getEstat().equals("C"));
		assertTrue(newActa.getRebut().getTpcobro().equals("E"));
	}
	
	
	@Test
	public void testFiltrarDocumento( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Response response = jerseyTest.target("acta/filterByDocument").queryParam("document", "12345678Z").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();
		List<Acta> actes=response.readEntity(new GenericType<List<Acta>>() {});
		
		assertEquals(200, response.getStatus());
		assertEquals(true, actes.size()>0);
	}	

	@Test
	public void testFiltrarNumActa( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Calendar calendar = new GregorianCalendar();
		calendar.setTimeInMillis(System.currentTimeMillis());
		Integer year=calendar.get(Calendar.YEAR);		
		Response response = jerseyTest.target("acta/filterByNumActa").queryParam("year", year.toString()).queryParam("numacta", "1").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();		List<Acta> actes=response.readEntity(new GenericType<List<Acta>>() {});
		
		assertEquals(200, response.getStatus());
		assertEquals(true, actes.size()>0);
	}	
	
	
	
	private static void initData() {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Response response = jerseyTest.target("persona/findById/1").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();		
		pers=response.readEntity(Persona.class);

		Response response2 = jerseyTest.target("article/1").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();		
		art=response2.readEntity(Article.class);

	}

}
