package test.endpoints;

import static org.junit.Assert.assertEquals;

import java.util.List;

import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.test.JerseyTest;
import org.glassfish.jersey.test.TestProperties;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import entity.security.Role;
import entity.security.Usuari;
import jakarta.ws.rs.client.Entity;
import jakarta.ws.rs.core.Application;
import jakarta.ws.rs.core.GenericType;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import jaxb.security.AuthDTO;
import jaxb.security.CredentialsDTO;
import rest.security.Authentication;
import rest.security.User;
import utilities.Config;

public class UsersTest {
	private static JerseyTest jerseyTest;
	

    @BeforeClass
    public static void beforeClass() throws Exception {
    	Config.persistentUnitName="TestSancionesApi";
    	initJerseyTest(); 
        jerseyTest.setUp();
    }	
    @AfterClass
    public static void afterClass() throws Exception {
    	jerseyTest.tearDown();
    }
	
	
    private static void initJerseyTest() {
        jerseyTest = new JerseyTest() {
        	@Override
            protected Application configure() {
                // Find first available port.
                enable(TestProperties.LOG_TRAFFIC);
                enable(TestProperties.DUMP_ENTITY); 
                return new ResourceConfig(Authentication.class, User.class);
            }
        };
      }    
    
	@Test
	public void testAnadirUsuario( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Role role=new Role();
		role.setKrole(2);
		role.setNom("ADMINISTRATIU");
		Usuari user=new Usuari();
		user.setEntrada("JUAN");
		user.setNomcomplet("JUAN");
		user.setCanviPassword(false);
		user.setPassword("Juan_100");
		user.setRole(role);
		Entity<Usuari> userEntity = Entity.entity(user, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("usuari").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .post(userEntity);
		Usuari newUser=response.readEntity(Usuari.class);
		assertEquals(200, response.getStatus());		
		assertEquals("",newUser.getPassword());
	}
	
	@Test
	public void testCambioPassword( ) {
		CredentialsDTO credentials=new CredentialsDTO("JUAN","Juan_100");
		AuthDTO auth=TokenFactory.makeAuth(credentials,jerseyTest);
		credentials.setNewpassword("Joan_200");
		Entity<CredentialsDTO> credentialsEntity = Entity.entity(credentials, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("usuari/changePassword").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .post(credentialsEntity);
		assertEquals(200, response.getStatus());		
	}
	

	@Test
	public void testModificarUsuario( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Role role=new Role();
		role.setKrole(2);
		role.setNom("ADMINISTRATIU");
		Usuari user=new Usuari();
		user.setKusuari(2);
		user.setNomcomplet("MARTA TOUS");
		user.setCanviPassword(false);
		user.setPassword("");
		user.setRole(role);		
		Entity<Usuari> userEntity = Entity.entity(user, MediaType.APPLICATION_JSON_TYPE);
		Response response = jerseyTest.target("usuari").request(MediaType.APPLICATION_JSON_TYPE)
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .put(userEntity);
		Usuari newUser=response.readEntity(Usuari.class);	
		assertEquals(200, response.getStatus());
		assertEquals("MARTA TOUS", newUser.getNomcomplet());
	}
	
	@Test
	public void testFiltrarUsuario( ) {
		AuthDTO auth=TokenFactory.makeAuth(new CredentialsDTO("admin","Admin2022"),jerseyTest);
		Response response = jerseyTest.target("usuari/list").queryParam("filter", "MARTA").request()
				.header("Authorization", "Bearer " + auth.getAccess_token())
		        .get();
		List<Usuari> users=response.readEntity(new GenericType<List<Usuari>>() {
        });
		
		assertEquals(200, response.getStatus());
		assertEquals(true, users.size()>0);
	}
	

	
	
}
